@@ -556,15 +556,21 @@ class MeasurementSettings(QWidget):
                 return
             lw = MeasurementListWidgetItem(Node(op="/", left=self.chosen_element_area, right=leaf))
             lw.setToolTip("User defined")
-            self.profile_options_chosen.addItem(lw)
+            self._add_option(lw)
             self.chosen_element.setIcon(QIcon())
             self.chosen_element = None
             self.chosen_element_area = None
-            if self.good_name():
-                self.save_butt.setEnabled(True)
-                self.save_butt_with_name.setEnabled(True)
-            if self.profile_options.count() == 0:
-                self.choose_butt.setDisabled(True)
+
+    def _add_option(self, item: MeasurementListWidgetItem):
+        for i in range(self.profile_options_chosen.count()):
+            if item.text() == self.profile_options_chosen.item(i).text():
+                return
+        self.profile_options_chosen.addItem(item)
+        if self.good_name():
+            self.save_butt.setEnabled(True)
+            self.save_butt_with_name.setEnabled(True)
+        if self.profile_options.count() == 0:
+            self.choose_butt.setDisabled(True)
 
     def create_selection_chosen_changed(self):
         # print(self.profile_options_chosen.count())
@@ -585,9 +591,7 @@ class MeasurementSettings(QWidget):
             self.move_down.setDisabled(True)
 
     def good_name(self):
-        if str(self.profile_name.text()).strip() == "":
-            return False
-        return True
+        return str(self.profile_name.text()).strip() != ""
 
     def move_down_fun(self):
         row = self.profile_options_chosen.currentRow()
@@ -645,16 +649,8 @@ class MeasurementSettings(QWidget):
         if node is None:
             return
         lw = MeasurementListWidgetItem(node)
-        for i in range(self.profile_options_chosen.count()):
-            if lw.text() == self.profile_options_chosen.item(i).text():
-                return
         lw.setToolTip(selected_item.toolTip())
-        self.profile_options_chosen.addItem(lw)
-        if self.good_name():
-            self.save_butt.setEnabled(True)
-            self.save_butt_with_name.setEnabled(True)
-        if self.profile_options.count() == 0:
-            self.choose_butt.setDisabled(True)
+        self._add_option(lw)
 
     def discard_option(self):
         selected_item: MeasurementListWidgetItem = self.profile_options_chosen.currentItem()
@@ -809,10 +805,7 @@ class SegAdvancedWindow(AdvancedWindow):
     def __init__(self, settings, parent=None, reload_list=None):
         super().__init__(settings, ["result_image", "raw_image"], parent)
         self.settings = settings
-        if reload_list is not None:
-            self.reload_list = reload_list
-        else:
-            self.reload_list = []
+        self.reload_list = reload_list if reload_list is not None else []
         self.setWindowTitle("Settings and Measurement")
         self.advanced_settings = Properties(settings)
         # self.colormap_settings = ColorSelector(settings, ["result_control"])
