@@ -1,4 +1,3 @@
-# coding=utf-8
 import logging
 import multiprocessing
 import os
@@ -155,9 +154,9 @@ class ProgressView(QWidget):
                 working_search = False
             if i < self.task_que.count():
                 item = self.task_que.item(i)
-                item.setText("Task {} ({}/{})".format(i, progress, total))
+                item.setText(f"Task {i} ({progress}/{total})")
             else:
-                self.task_que.addItem("Task {} ({}/{})".format(i, progress, total))
+                self.task_que.addItem(f"Task {i} ({progress}/{total})")
         if not self.calculation_manager.has_work:
             self.part_progress.setValue(self.part_progress.maximum())
             self.preview_timer.stop()
@@ -315,6 +314,7 @@ class BatchWindow(QTabWidget):
             else:
                 event.ignore()
         self.settings.set_in_profile("batch_window_geometry", self.saveGeometry().toHex().data().decode("ascii"))
+        super().closeEvent(event)
 
 
 class CalculationPrepare(QDialog):
@@ -367,17 +367,15 @@ class CalculationPrepare(QDialog):
 
         self.mask_path_list = []
         self.mask_mapper_list = self.calculation_plan.get_list_file_mask()
-        mask_file_list = []
-        for i, el in enumerate(self.mask_mapper_list):
-            if isinstance(el, MaskFile):
-                mask_file_list.append((i, el))
+        mask_file_list = [(i, el) for i, el in enumerate(self.mask_mapper_list) if isinstance(el, MaskFile)]
+
         mask_path_layout = QGridLayout()
         for i, (pos, mask_file) in enumerate(mask_file_list):
             if mask_file.name == "":
-                mask_path_layout.addWidget(right_label("Path to file {} with mask mapping".format(i + 1)))
+                mask_path_layout.addWidget(right_label(f"Path to file {i + 1} with mask mapping"))
             else:
                 mask_path_layout.addWidget(
-                    right_label("Path to file {} with mask mapping for name: {}".format(i + 1, mask_file.name))
+                    right_label(f"Path to file {i + 1} with mask mapping for name: {mask_file.name}")
                 )
             mask_path = QLineEdit(self)
             mask_path.setReadOnly(True)
@@ -517,7 +515,7 @@ class CalculationPrepare(QDialog):
                     exist = os.path.exists(mask_path)
                     if exist:
                         sub_widget = QTreeWidgetItem(widget)
-                        sub_widget.setText(0, "Mask {} ok".format(mask_mapper.name))
+                        sub_widget.setText(0, f"Mask {mask_mapper.name} ok")
                         sub_widget.setIcon(0, ok_icon)
                         self.state_list[file_num, mask_num] = 0
                     else:
@@ -532,14 +530,10 @@ class CalculationPrepare(QDialog):
                         self.state_list[file_num, mask_num] = 2
                 else:
                     sub_widget = QTreeWidgetItem(widget)
-                    sub_widget.setText(0, "Mask {} unknown".format(mask_mapper.name))
+                    sub_widget.setText(0, f"Mask {mask_mapper.name} unknown")
                     sub_widget.setIcon(0, warn_icon)
                     self.state_list[file_num, mask_num] = 1
-            if self.state_list.shape[1] == 0:
-                state = 0
-            else:
-                state = self.state_list[file_num].max()
-
+            state = 0 if self.state_list.shape[1] == 0 else self.state_list[file_num].max()
             if state == 0:
                 widget.setIcon(0, ok_icon)
             elif state == 1:
