@@ -30,10 +30,10 @@ from PartSegCore.analysis import ProjectTuple, algorithm_description, load_funct
 from PartSegCore.analysis.analysis_utils import SegmentationPipeline, SegmentationPipelineElement
 from PartSegCore.analysis.io_utils import create_history_element_from_project
 from PartSegCore.analysis.save_functions import save_dict
-from PartSegCore.io_utils import HistoryElement, WrongFileTypeException
-from PartSegCore.mask_create import calculate_mask_from_project
+from PartSegCore.io_utils import WrongFileTypeException
+from PartSegCore.project_info import HistoryElement, calculate_mask_from_project
 from PartSegCore.roi_info import ROIInfo
-from PartSegCore.segmentation.algorithm_base import SegmentationResult
+from PartSegCore.segmentation.algorithm_base import ROIExtractionResult
 from PartSegImage import TiffImageReader
 
 from ..common_gui.algorithms_description import AlgorithmChoose, InteractiveAlgorithmSettingsWidget
@@ -109,8 +109,6 @@ class Options(QWidget):
 
         self.label = TextShow()
 
-        # self.label.setWordWrap(True)
-        # self.label.setTextInteractionFlags(Qt.TextSelectableByMouse)
         layout = QVBoxLayout()
         layout2 = QHBoxLayout()
         layout2.setSpacing(1)
@@ -164,8 +162,8 @@ class Options(QWidget):
             mask = el.mask_property
             segmentation = ROIExtractionProfile(
                 name="Unknown",
-                algorithm=el.segmentation_parameters["algorithm_name"],
-                values=el.segmentation_parameters["values"],
+                algorithm=el.roi_extraction_parameters["algorithm_name"],
+                values=el.roi_extraction_parameters["values"],
             )
             new_el = SegmentationPipelineElement(mask_property=mask, segmentation=segmentation)
             mask_history.append(new_el)
@@ -183,15 +181,14 @@ class Options(QWidget):
             text, ok = QInputDialog.getText(self, "Pipeline name", "Input pipeline name here")
             if not ok:
                 return
-            if text in self._settings.segmentation_pipelines:
-                if QMessageBox.No == QMessageBox.warning(
-                    self,
-                    "Already exists",
-                    "Profile with this name already exist. Overwrite?",
-                    QMessageBox.Yes | QMessageBox.No,
-                    QMessageBox.No,
-                ):
-                    continue
+            if text in self._settings.segmentation_pipelines and QMessageBox.No == QMessageBox.warning(
+                self,
+                "Already exists",
+                "Profile with this name already exist. Overwrite?",
+                QMessageBox.Yes | QMessageBox.No,
+                QMessageBox.No,
+            ):
+                continue
             profile = SegmentationPipeline(name=text, segmentation=current_segmentation, mask_history=mask_history)
             self._settings.segmentation_pipelines[text] = profile
             self._settings.dump()
@@ -208,7 +205,7 @@ class Options(QWidget):
         if dial.exec() and process_thread.result:
             pipeline_result = process_thread.result
             self._settings.mask = pipeline_result.mask
-            self._settings.roi = pipeline_result.roi
+            self._settings.roi = pipeline_result.roi_info.roi
             self._settings.set_history(pipeline_result.history)
             self.label.setText(pipeline_result.description)
             self.algorithm_choose_widget.change_algorithm(pipeline.segmentation.algorithm, pipeline.segmentation.values)
@@ -256,7 +253,7 @@ class Options(QWidget):
         return super().event(event)
 
     def keyPressEvent(self, event: QKeyEvent):
-        if (event.key() == Qt.Key_Enter or event.key() == Qt.Key_Return) and (event.modifiers() == Qt.ControlModifier):
+        if event.key() in [Qt.Key_Enter, Qt.Key_Return] and event.modifiers() == Qt.ControlModifier:
             self.execute_btn.click()
 
     def save_profile(self):
@@ -265,15 +262,14 @@ class Options(QWidget):
             text, ok = QInputDialog.getText(self, "Profile Name", "Input profile name here")
             if not ok:
                 return
-            if text in self._settings.segmentation_profiles:
-                if QMessageBox.No == QMessageBox.warning(
-                    self,
-                    "Already exists",
-                    "Profile with this name already exist. Overwrite?",
-                    QMessageBox.Yes | QMessageBox.No,
-                    QMessageBox.No,
-                ):
-                    continue
+            if text in self._settings.segmentation_profiles and QMessageBox.No == QMessageBox.warning(
+                self,
+                "Already exists",
+                "Profile with this name already exist. Overwrite?",
+                QMessageBox.Yes | QMessageBox.No,
+                QMessageBox.No,
+            ):
+                continue
             resp = ROIExtractionProfile(text, widget.name, widget.get_values())
             self._settings.segmentation_profiles[text] = resp
             self._settings.dump()
@@ -329,12 +325,12 @@ class Options(QWidget):
         widget: InteractiveAlgorithmSettingsWidget = self.algorithm_choose_widget.current_widget()
         if self._settings.image.is_time and not widget.algorithm.support_time():
             QMessageBox.information(
-                self, "Not supported", "This algorithm do not support time data. " "You can convert it in image adjust"
+                self, "Not supported", "This algorithm do not support time data. You can convert it in image adjust"
             )
             return
         if self._settings.image.is_stack and not widget.algorithm.support_z():
             QMessageBox.information(
-                self, "Not supported", "This algorithm do not support stack data. " "You can convert it in image adjust"
+                self, "Not supported", "This algorithm do not support stack data. You can convert it in image adjust"
             )
             return
         self._settings.last_executed_algorithm = widget.name
@@ -342,7 +338,7 @@ class Options(QWidget):
         self.interactive_use.setDisabled(True)
         widget.execute()
 
-    def execution_done(self, segmentation: SegmentationResult):
+    def execution_done(self, segmentation: ROIExtractionResult):
         if segmentation.info_text != "":
             QMessageBox.information(self, "Algorithm info", segmentation.info_text)
         self._settings.set_segmentation_result(segmentation)
@@ -492,18 +488,18 @@ class MainMenu(BaseMainMenu):
                     self.set_data(result)
 
         except ValueError as e:
-            QMessageBox.warning(self, "Open error", "{}".format(e))
+            QMessageBox.warning(self, "Open error", f"{e}")
 
     def batch_window(self):
-        if self.main_window.batch_window is not None:
-            if self.main_window.batch_window.isVisible():
-                self.main_window.batch_window.activateWindow()
-            else:
-                self.main_window.batch_window.show()
-        else:
+        if self.main_window.batch_window is None:
             self.main_window.batch_window = BatchWindow(self.settings)
             self.main_window.batch_window.show()
 
+        elif self.main_window.batch_window.isVisible():
+            self.main_window.batch_window.activateWindow()
+        else:
+            self.main_window.batch_window.show()
+
     def advanced_window_show(self):
         if self.main_window.advanced_window.isVisible():
             self.main_window.advanced_window.activateWindow()
@@ -512,6 +508,7 @@ class MainMenu(BaseMainMenu):
 
 
 class MaskDialog(MaskDialogBase):
+    # FIXME add tests
     def next_mask(self):
         project_info: ProjectTuple = self.settings.get_project_info()
         mask_property = self.mask_widget.get_mask_property()
@@ -525,10 +522,10 @@ class MaskDialog(MaskDialogBase):
         )
         if self.settings.history_redo_size():
             history: HistoryElement = self.settings.history_next_element()
-            self.settings.set("current_algorithm", history.segmentation_parameters["algorithm_name"])
+            self.settings.set("current_algorithm", history.roi_extraction_parameters["algorithm_name"])
             self.settings.set(
-                f"algorithm.{history.segmentation_parameters['algorithm_name']}",
-                history.segmentation_parameters["values"],
+                f"algorithm.{history.roi_extraction_parameters['algorithm_name']}",
+                history.roi_extraction_parameters["values"],
             )
         self.settings.mask = mask
         self.close()
@@ -538,19 +535,12 @@ class MaskDialog(MaskDialogBase):
         algorithm_name = self.settings.last_executed_algorithm
         algorithm_values = self.settings.get(f"algorithms.{algorithm_name}")
         self.settings.fix_history(algorithm_name=algorithm_name, algorithm_values=algorithm_values)
-        self.settings.set("current_algorithm", history.segmentation_parameters["algorithm_name"])
+        self.settings.set("current_algorithm", history.roi_extraction_parameters["algorithm_name"])
         self.settings.set(
-            f"algorithm.{history.segmentation_parameters['algorithm_name']}", history.segmentation_parameters["values"]
+            f"algorithm.{history.roi_extraction_parameters['algorithm_name']}",
+            history.roi_extraction_parameters["values"],
         )
-        history.arrays.seek(0)
-        seg = np.load(history.arrays)
-        history.arrays.seek(0)
-        self.settings.roi = seg["segmentation"]
-        self.settings.full_segmentation = seg["full_segmentation"]
-        if "mask" in seg:
-            self.settings.mask = seg["mask"]
-        else:
-            self.settings.mask = None
+        self.settings.roi, self.settings.mask = history.get_roi_info_and_mask()
         self.close()
 
 
@@ -598,10 +588,7 @@ class MainWindow(BaseMainWindow):
             im = reader.read(self.initial_image_path)
             im.file_path = ""
             self.settings.image = im
-        elif initial_image is False:
-            # FIXME This is for test opening
-            pass
-        else:
+        elif initial_image is not False:
             self.settings.image = initial_image
 
         icon = QIcon(os.path.join(PartSegData.icons_dir, "icon.png"))
@@ -677,7 +664,7 @@ class MainWindow(BaseMainWindow):
                 ret = QMessageBox.warning(
                     self,
                     "Batch work",
-                    "Batch work is not finished. " "Would you like to terminate it?",
+                    "Batch work is not finished. Would you like to terminate it?",
                     QMessageBox.No | QMessageBox.Yes,
                 )
                 if ret == QMessageBox.Yes:
