@@ -11,8 +11,9 @@ from PartSegCore.analysis.io_utils import MaskInfo, ProjectTuple
 from PartSegCore.analysis.load_functions import load_metadata
 from PartSegCore.analysis.measurement_calculation import MeasurementProfile
 from PartSegCore.analysis.save_hooks import PartEncoder
-from PartSegCore.io_utils import HistoryElement
+from PartSegCore.io_utils import PointsInfo
 from PartSegCore.json_hooks import ProfileDict
+from PartSegCore.project_info import HistoryElement
 from PartSegCore.roi_info import ROIInfo
 
 from ..common_backend.base_settings import BaseSettings, SaveSettingsDescription
@@ -60,7 +61,7 @@ class PartSettings(BaseSettings):
         :param dict algorithm_values:
         """
         self.history[self.history_index + 1] = self.history[self.history_index + 1].replace_(
-            segmentation_parameters={"algorithm_name": algorithm_name, "values": algorithm_values}
+            roi_extraction_parameters={"algorithm_name": algorithm_name, "values": algorithm_values}
         )
 
     @staticmethod
@@ -71,13 +72,6 @@ class PartSettings(BaseSettings):
         self.compare_segmentation = segmentation
         self.compare_segmentation_change.emit(segmentation)
 
-    @property
-    def use_physical_unit(self):
-        return self.get("use_physical_unit", False)
-
-    def set_use_physical_unit(self, value):
-        self.set("use_physical_unit", value)
-
     def _image_changed(self):
         super()._image_changed()
         self._mask = None
@@ -94,36 +88,42 @@ class PartSettings(BaseSettings):
         return ProjectTuple(
             file_path=self.image.file_path,
             image=self.image.substitute(),
-            roi=self.roi,
             roi_info=self.roi_info,
             additional_layers=self.additional_layers,
             mask=self.mask,
             history=self.history[: self.history_index + 1],
             algorithm_parameters=algorithm_val,
+            points=self.points,
         )
 
-    def set_project_info(self, data: typing.Union[ProjectTuple, MaskInfo]):
-        if isinstance(data, ProjectTuple):
-            if self.image.file_path == data.image.file_path and self.image.shape == data.image.shape:
-                if data.roi is not None:
-                    try:
-                        self.image.fit_array_to_image(data.roi)
-                        self.mask = data.mask
-                    except ValueError:
-                        self.image = data.image.substitute()
-                else:
-                    self.mask = data.mask
-            else:
-                self.image = data.image.substitute(mask=data.mask)
-            self.roi = data.roi
-            self._additional_layers = data.additional_layers
-            self.set_history(data.history[:])
-            if data.algorithm_parameters:
-                self.last_executed_algorithm = data.algorithm_parameters["algorithm_name"]
-                self.set(f"algorithms.{self.last_executed_algorithm}", deepcopy(data.algorithm_parameters["values"]))
-                self.algorithm_changed.emit()
-        elif isinstance(data, MaskInfo):
+    def set_project_info(self, data: typing.Union[ProjectTuple, MaskInfo, PointsInfo]):
+        if isinstance(data, MaskInfo):
             self.mask = data.mask_array
+            return
+        if isinstance(data, PointsInfo):
+            self.points = data.points
+            return
+        if not isinstance(data, ProjectTuple):
+            return
+        if self.image.file_path == data.image.file_path and self.image.shape == data.image.shape:
+            if data.roi_info.roi is not None:
+                try:
+                    self.image.fit_array_to_image(data.roi_info.roi)
+                    self.mask = data.mask
+                except ValueError:
+                    self.image = data.image.substitute()
+            else:
+                self.mask = data.mask
+        else:
+            self.image = data.image.substitute(mask=data.mask)
+        self.roi = data.roi_info
+        self._additional_layers = data.additional_layers
+        self.additional_layers_changed.emit()
+        self.set_history(data.history[:])
+        if data.algorithm_parameters:
+            self.last_executed_algorithm = data.algorithm_parameters["algorithm_name"]
+            self.set(f"algorithms.{self.last_executed_algorithm}", deepcopy(data.algorithm_parameters["values"]))
+            self.algorithm_changed.emit()
 
     def get_save_list(self) -> typing.List[SaveSettingsDescription]:
         return super().get_save_list() + [
