@@ -678,8 +678,6 @@ class CreatePlan(QWidget):
                 self.calculation_plan.replace_step(profile)
             else:
                 self.calculation_plan.add_step(profile)
-            self.plan.update_view()
-
         else:  # self.segment_stack.currentIndex() == 1
             text = self.pipeline_profile.currentItem().text()
             segmentation_pipeline = self.settings.segmentation_pipelines[text]
@@ -697,7 +695,8 @@ class CreatePlan(QWidget):
                 self.calculation_plan.set_position(pos)
             self.calculation_plan.add_step(segmentation_pipeline.segmentation)
             self.calculation_plan.set_position(old_pos)
-            self.plan.update_view()
+
+        self.plan.update_view()
 
     def add_measurement(self):
         text = str(self.measurements_list.currentItem().text())
@@ -743,7 +742,7 @@ class CreatePlan(QWidget):
             if self.mask_allow and (name == "" or name not in self.mask_set):
                 self.generate_mask_btn.setEnabled(True)
         else:
-            if self.node_type != NodeType.file_mask and self.node_type != NodeType.mask:
+            if self.node_type not in [NodeType.file_mask, NodeType.mask]:
                 return
             # generate mask from segmentation
             if self.node_type == NodeType.mask and (name == "" or name == self.node_name or name not in self.mask_set):
@@ -754,7 +753,7 @@ class CreatePlan(QWidget):
             text, ok = QInputDialog.getText(self, "Plan title", "Set plan title")
         else:
             text, ok = QInputDialog.getText(
-                self, "Plan title", "Set plan title. Previous ({}) is already in use".format(text), text=text
+                self, "Plan title", f"Set plan title. Previous ({text}) is already in use", text=text
             )
         text = text.strip()
         if ok:
@@ -893,7 +892,7 @@ class PlanPreview(QTreeWidget):
     changed_node = Signal()
 
     def __init__(self, parent=None, calculation_plan=None):
-        super(PlanPreview, self).__init__(parent)
+        super().__init__(parent)
         self.calculation_plan = calculation_plan
         self.header().close()
         self.itemSelectionChanged.connect(self.set_path)
@@ -1009,7 +1008,7 @@ class PlanPreview(QTreeWidget):
                         QTreeWidgetItem(child, [line])
 
             else:
-                logging.error("Unknown operation {}".format(op_type))
+                logging.error(f"Unknown operation {op_type}")
         self.blockSignals(False)
         self.set_path()
         self.changed_node.emit()
@@ -1024,7 +1023,7 @@ class CalculateInfo(QWidget):
     plan_to_edit_signal = Signal()
 
     def __init__(self, settings: PartSettings):
-        super(CalculateInfo, self).__init__()
+        super().__init__()
         self.settings = settings
         self.calculate_plans = SearchableListWidget(self)
         self.plan_view = PlanPreview(self)
