@@ -1,6 +1,5 @@
 from typing import List
 
-import numpy as np
 from napari.layers import Layer
 from vispy.app import MouseEvent
 
@@ -51,9 +50,9 @@ class StackImageView(ImageView):
             self.additional_layers[-1].translate_grid = translate_grid
 
     def component_click(self, _event: MouseEvent):
-        if self.viewer.active_layer is None:
+        cords = self._coordinates()
+        if cords is None:
             return
-        cords = np.array([int(x) for x in self.viewer.active_layer.coordinates])
         for image_info in self.image_info.values():
             if image_info.roi_info.roi is None:
                 continue
@@ -69,9 +68,9 @@ class StackImageView(ImageView):
         text_list = []
         for el in self.components:
             if self.settings.component_is_chosen(el):
-                text_list.append("☑{}".format(el))
+                text_list.append(f"☑{el}")
             else:
-                text_list.append("☐{}".format(el))
+                text_list.append(f"☐{el}")
         if text:
             return " ".join(text_list) + "\n" + text
         return " ".join(text_list)
