@@ -1,6 +1,4 @@
 import os
-from collections import defaultdict
-from copy import deepcopy
 from functools import partial
 from pathlib import Path
 from typing import Type
@@ -30,19 +28,12 @@ from qtpy.QtWidgets import (
 
 import PartSegData
 from PartSegCore import UNIT_SCALE, Units, state_store
-from PartSegCore.io_utils import HistoryElement, HistoryProblem, WrongFileTypeException
+from PartSegCore.io_utils import WrongFileTypeException
 from PartSegCore.mask import io_functions
 from PartSegCore.mask.algorithm_description import mask_algorithm_dict
 from PartSegCore.mask.history_utils import create_history_element_from_segmentation_tuple
-from PartSegCore.mask.io_functions import (
-    LoadROIFromTIFF,
-    LoadROIParameters,
-    LoadSegmentation,
-    MaskProjectTuple,
-    SaveROI,
-)
-from PartSegCore.mask_create import calculate_mask_from_project
-from PartSegCore.segmentation.algorithm_base import SegmentationResult
+from PartSegCore.mask.io_functions import LoadROI, LoadROIFromTIFF, LoadROIParameters, MaskProjectTuple, SaveROI
+from PartSegCore.project_info import HistoryElement, HistoryProblem, calculate_mask_from_project
 from PartSegImage import Image, TiffImageReader
 
 from .._roi_mask.segmentation_info_dialog import SegmentationInfoDialog
@@ -69,6 +60,7 @@ CONFIG_FOLDER = os.path.join(state_store.save_folder, "mask")
 
 
 class MaskDialog(MaskDialogBase):
+    # FIXME add tests
     def __init__(self, settings: StackSettings):
         super().__init__(settings)
         self.settings = settings
@@ -98,13 +90,10 @@ class MaskDialog(MaskDialogBase):
         self.settings.set_segmentation(
             seg["segmentation"],
             False,
-            history.segmentation_parameters["selected"],
-            history.segmentation_parameters["parameters"],
+            history.roi_extraction_parameters["selected"],
+            history.roi_extraction_parameters["parameters"],
         )
-        if "mask" in seg:
-            self.settings.mask = seg["mask"]
-        else:
-            self.settings.mask = None
+        self.settings.mask = seg["mask"] if "mask" in seg else None
         self.close()
 
 
@@ -183,7 +172,7 @@ class MainMenu(BaseMainMenu):
         default_file_path = self.settings.get("io.load_image_file", "")
         if os.path.isfile(default_file_path):
             dial.selectFile(default_file_path)
-        dial.selectNameFilter(self.settings.get("io.load_data_filter", next(iter(io_functions.load_dict.keys()))))
+        dial.selectNameFilter(self.settings.get("io.load_data_filter", io_functions.load_dict.get_default()))
         dial.setHistory(dial.history() + self.settings.get_path_history())
         if not dial.exec_():
             return
@@ -248,7 +237,7 @@ class MainMenu(BaseMainMenu):
     def load_segmentation(self):
         dial = CustomLoadDialog(
             {
-                LoadSegmentation.get_name(): LoadSegmentation,
+                LoadROI.get_name(): LoadROI,
                 LoadROIParameters.get_name(): LoadROIParameters,
                 LoadROIFromTIFF.get_name(): LoadROIFromTIFF,
             }
@@ -417,7 +406,7 @@ class ChosenComponents(QWidget):
     mouse_leave = Signal(int)
 
     def __init__(self):
-        super(ChosenComponents, self).__init__()
+        super().__init__()
         # self.setLayout(FlowLayout())
         self.check_box = {}
         self.check_all_btn = QPushButton("Select all")
@@ -488,11 +477,7 @@ class ChosenComponents(QWidget):
         return self.check_box[num].isChecked()
 
     def get_chosen(self):
-        res = []
-        for num, check in self.check_box.items():
-            if check.isChecked():
-                res.append(num)
-        return res
+        return [num for num, check in self.check_box.items() if check.isChecked()]
 
     def get_mask(self):
         res = [0]
@@ -532,7 +517,7 @@ class AlgorithmOptions(QWidget):
         self.save_parameters_btn = QPushButton("Save parameters")
         self.block_execute_all_btn = False
         self.algorithm_choose_widget = AlgorithmChoose(settings, mask_algorithm_dict)
-        self.algorithm_choose_widget.result.connect(self.execution_done)
+        self.algorithm_choose_widget.result.connect(self.execution_result_set)
         self.algorithm_choose_widget.finished.connect(self.execution_finished)
         self.algorithm_choose_widget.progress_signal.connect(self.progress_info)
 
@@ -759,16 +744,8 @@ class AlgorithmOptions(QWidget):
         self.progress_info_lab.setHidden(True)
         self.choose_components.setDisabled(False)
 
-    def execution_done(self, segmentation: SegmentationResult):
-        if np.max(segmentation.roi) == 0:
-            QMessageBox.information(
-                self, "No result", "Segmentation contains no component, check parameters, " "especially chosen channel."
-            )
-        if segmentation.info_text != "":
-            QMessageBox.information(self, "Algorithm info", segmentation.info_text)
-        parameters_dict = defaultdict(lambda: deepcopy(segmentation.parameters))
-        self.settings.additional_layers = segmentation.additional_layers
-        self.settings.set_segmentation(segmentation.roi, True, [], parameters_dict)
+    def execution_result_set(self, result):
+        self.settings.set_segmentation_result(result)
 
     def showEvent(self, _):
         widget = self.algorithm_choose_widget.current_widget()
@@ -778,7 +755,7 @@ class AlgorithmOptions(QWidget):
 class ImageInformation(QWidget):
     def __init__(self, settings: StackSettings, parent=None):
         """:type settings: ImageSettings"""
-        super(ImageInformation, self).__init__(parent)
+        super().__init__(parent)
         self._settings = settings
         self.path = QTextEdit("<b>Path:</b> example image")
         self.path.setWordWrapMode(QTextOption.WrapAnywhere)
@@ -837,7 +814,7 @@ class ImageInformation(QWidget):
             self.spacing[2].setDisabled(False)
 
     def set_image_path(self, value):
-        self.path.setText("<b>Path:</b> {}".format(value))
+        self.path.setText(f"<b>Path:</b> {value}")
         self.update_spacing()
 
     def image_spacing_change(self):
@@ -951,10 +928,7 @@ class MainWindow(BaseMainWindow):
             im = reader.read(self.initial_image_path)
             im.file_path = ""
             self.settings.image = im
-        elif initial_image is False:
-            # FIXME This is for test opening
-            pass
-        else:
+        elif initial_image is not False:
             self.settings.image = initial_image
         try:
             geometry = self.settings.get_from_profile("main_window_geometry")
