@@ -1,6 +1,68 @@
 changelog = """
 # Changelog
 
+## 0.13.7
+
+-   add measurements widget for napari
+-   fix bug in pipeline usage
+
+## 0.13.6
+
+-   Hotfix release
+-   Prepare for a new napari version
+
+## 0.13.5
+
+-   Small fixes for error reporting
+-   Fix mask segmentation
+
+## 0.13.4
+
+-   Bugfix for outdated profile/pipeline preview
+
+## 0.13.3
+
+-   Fix saving roi_info in multiple files and history
+
+## 0.13.2
+
+-   Fix showing label in select label tab
+
+## 0.13.1
+
+-   Add Haralick measurements
+-   Add obsep file support
+
+## 0.13.0
+
+-   Add possibility of custom input widgets for algorithms
+-   Switch to napari Colormaps instead of custom one
+-   Add points visualization
+-   Synchronization widget for builtin (View menu) napari viewer
+-   Drop Python 3.6
+
+## 0.12.7
+
+-   Fixes for napari 0.4.6
+
+## 0.12.6
+
+-   Fix prev_mask_get
+-   Fix cache mechanism on mask change
+-   Update PyInstaller build
+
+## 0.12.5
+
+-   Fix bug in pipeline execute
+
+## 0.12.4
+
+-   Fix ROI Mask windows related build (signal not properly connected)
+
+## 0.12.3
+
+-   Fix ROI Mask
+
 ## 0.12.2
 
 -   Fix windows bundle
