@@ -9,21 +9,45 @@ from typing import Any, Dict, List, NamedTuple, Optional, Tuple, Union
 
 import napari.utils.theme
 import numpy as np
-from napari.resources import get_stylesheet
+from napari.utils import Colormap
 from napari.utils.theme import template as napari_template
 from qtpy.QtCore import QObject, Signal
 from qtpy.QtWidgets import QMessageBox, QWidget
 
 from PartSeg.common_backend.partially_const_dict import PartiallyConstDict
-from PartSegCore.color_image import ColorMap, default_colormap_dict, default_label_dict
+from PartSegCore import register
+from PartSegCore.color_image import default_colormap_dict, default_label_dict
 from PartSegCore.color_image.base_colors import starting_colors
-from PartSegCore.io_utils import HistoryElement, load_metadata_base
+from PartSegCore.io_utils import load_metadata_base
 from PartSegCore.json_hooks import ProfileDict, ProfileEncoder, check_loaded_dict
-from PartSegCore.project_info import ProjectInfoBase
+from PartSegCore.project_info import AdditionalLayerDescription, HistoryElement, ProjectInfoBase
 from PartSegCore.roi_info import ROIInfo
-from PartSegCore.segmentation.algorithm_base import AdditionalLayerDescription, SegmentationResult
+from PartSegCore.segmentation.algorithm_base import ROIExtractionResult
 from PartSegImage import Image
 
+try:
+    from napari.qt import get_stylesheet
+except ImportError:  # pragma: no cover
+    from napari.resources import get_stylesheet
+
+if hasattr(napari.utils.theme, "get_theme"):
+
+    def get_theme(name: str) -> dict:
+        theme = napari.utils.theme.get_theme(name)
+        if "canvas" in theme and theme["canvas"] != "black":
+            theme["canvas"] = "black"
+            napari.utils.theme.register_theme(name, theme)
+        return theme
+
+
+else:  # pragma: no cover
+
+    def get_theme(name: str) -> dict:
+        theme = napari.utils.theme.palettes[name]
+        theme["canvas"] = "black"
+        return theme
+
+
 DIR_HISTORY = "io.dir_location_history"
 FILE_HISTORY = "io.files_open_history"
 
@@ -56,19 +80,19 @@ class ImageSettings(QObject):
         self._parent = parent
 
     @property
-    def full_segmentation(self):
+    def full_segmentation(self):  # pragma: no cover
         raise AttributeError("full_segmentation not supported")
 
     @full_segmentation.setter
-    def full_segmentation(self, val):  # pylint: disable=R0201
+    def full_segmentation(self, val):  # pragma: no cover # pylint: disable=R0201
         raise AttributeError("full_segmentation not supported")
 
     @property
-    def noise_remove_image_part(self):
+    def noise_remove_image_part(self):  # pragma: no cover
         raise AttributeError("noise_remove_image_part not supported")
 
     @noise_remove_image_part.setter
-    def noise_remove_image_part(self, val):  # pylint: disable=R0201
+    def noise_remove_image_part(self, val):  # pragma: no cover # pylint: disable=R0201
         raise AttributeError("noise_remove_image_part not supported")
 
     @property
@@ -76,7 +100,7 @@ class ImageSettings(QObject):
         return self._additional_layers
 
     @additional_layers.setter
-    def additional_layers(self, val):  # pylint: disable=R0201
+    def additional_layers(self, val):  # pragma: no cover  # pylint: disable=R0201
         raise AttributeError("additional_layers assign not supported")
 
     @property
@@ -92,7 +116,7 @@ class ImageSettings(QObject):
 
     @image_spacing.setter
     def image_spacing(self, value):
-        if len(value) not in [2, 3]:
+        if len(value) not in [2, 3]:  # pragma: no cover
             raise ValueError(f"value parameter should have length 2 or 3. Current length is {len(value)}.")
         if len(value) == 2:
             self._image.set_spacing(tuple([self._image.spacing[0]] + list(value)))
@@ -101,7 +125,7 @@ class ImageSettings(QObject):
         self.image_spacing_changed.emit()
 
     @property
-    def segmentation(self) -> np.ndarray:
+    def segmentation(self) -> np.ndarray:  # pragma: no cover
         """current roi"""
         warnings.warn("segmentation parameter is renamed to roi", DeprecationWarning)
         return self.roi
@@ -112,7 +136,7 @@ class ImageSettings(QObject):
         return self._roi_info.roi
 
     @property
-    def segmentation_info(self) -> ROIInfo:
+    def segmentation_info(self) -> ROIInfo:  # pragma: no cover
         warnings.warn("segmentation info parameter is renamed to roi", DeprecationWarning)
         return self.roi_info
 
@@ -131,8 +155,8 @@ class ImageSettings(QObject):
             if isinstance(val, np.ndarray):
                 self._roi_info = ROIInfo(self.image.fit_array_to_image(val))
             else:
-                self._roi_infol = val.fit_to_image(self.image)
-        except ValueError:
+                self._roi_info = val.fit_to_image(self.image)
+        except ValueError:  # pragma: no cover
             raise ValueError("roi do not fit to image")
         self._additional_layers = {}
         self.roi_changed.emit(self._roi_info)
@@ -160,11 +184,10 @@ class ImageSettings(QObject):
 
     @property
     def has_channels(self):
-        return self._image.channels > 1
+        return self.channels > 1
 
     def _image_changed(self):
         """Reimplement hook for change of main image"""
-        pass
 
     @property
     def image_path(self):
@@ -174,6 +197,7 @@ class ImageSettings(QObject):
 
     @property
     def image_shape(self):
+        # TODO analyse and decide if channels should be part of shape
         if self.image is not None:
             return self._image.shape
         return ()
@@ -193,12 +217,15 @@ class ImageSettings(QObject):
         return np.array([0] + [1] * np.max(self.roi), dtype=np.uint8)
 
 
-class ColormapDict(PartiallyConstDict[ColorMap]):
+class ColormapDict(PartiallyConstDict[Colormap]):
     """
     Dict for mixing custom colormap with predefined ones
     """
 
-    const_item_dict = default_colormap_dict
+    if os.path.basename(sys.argv[0]) in ["sphinx-build", "sphinx-build.exe"]:
+        const_item_dict = {}
+    else:
+        const_item_dict = default_colormap_dict
     """
     Non removable items for this dict. Current value is :py:data:`default_colormap_dict`
     """
@@ -254,9 +281,9 @@ class ViewSettings(ImageSettings):
 
     @property
     def style_sheet(self):
-        palette = napari.utils.theme.palettes[self.theme_name]
-        palette["canvas"] = "black"
-        return napari_template(get_stylesheet(), **palette)
+        theme = get_theme(self.theme_name)
+        # TODO understand qss overwrite mechanism
+        return napari_template("\n".join(register.qss_list) + get_stylesheet() + "\n".join(register.qss_list), **theme)
 
     @theme_name.setter
     def theme_name(self, value: str):
@@ -276,7 +303,7 @@ class ViewSettings(ImageSettings):
         data = self.get_from_profile("colormaps", starting_colors[:])
         res = [x for x in data if x in self.colormap_dict]
         if len(res) != data:
-            if len(res) == 0:
+            if not res:
                 res = starting_colors[:]
             self.set_in_profile("colormaps", res)
         return res
@@ -385,6 +412,7 @@ class BaseSettings(ViewSettings):
     """
 
     mask_changed = Signal()
+    points_changed = Signal()
     mask_representation_changed = Signal()
     request_load_files = Signal(list)
     """:py:class:`~.Signal` mask changed signal"""
@@ -404,8 +432,31 @@ class BaseSettings(ViewSettings):
         self.history: List[HistoryElement] = []
         self.history_index = -1
         self.last_executed_algorithm = ""
+        self._points = None
 
-    def set_segmentation_result(self, result: SegmentationResult):
+    def _image_changed(self):
+        super()._image_changed()
+        self.points = None
+
+    @property
+    def points(self):
+        return self._points
+
+    @points.setter
+    def points(self, value):
+        self._points = value if value is not None else None
+        self.points_changed.emit()
+
+    def set_segmentation_result(self, result: ROIExtractionResult):
+        if (
+            result.file_path is not None and result.file_path != "" and result.file_path != self.image.file_path
+        ):  # pragma: no cover
+            if self._parent is not None:
+                # TODO change to non disrupting popup
+                QMessageBox().warning(
+                    self._parent, "Result file bug", "It looks like one try to set ROI form another file."
+                )
+            return
         if result.info_text and self._parent is not None:
             QMessageBox().information(self._parent, "Algorithm info", result.info_text)
 
@@ -417,7 +468,7 @@ class BaseSettings(ViewSettings):
             alternative_list = {
                 k: self.image.fit_array_to_image(v) for k, v in result.alternative_representation.items()
             }
-        except ValueError:
+        except ValueError:  # pragma: no cover
             raise ValueError("roi do not fit to image")
         self._roi_info = ROIInfo(roi, result.roi_annotation, alternative_list)
         self.roi_changed.emit(self._roi_info)
@@ -517,7 +568,7 @@ class BaseSettings(ViewSettings):
         self.set(DIR_HISTORY, self._add_elem_to_list(self.get(DIR_HISTORY, []), dir_path))
 
     def add_load_files_history(self, file_path: List[str], load_method: str):
-        self.set(FILE_HISTORY, self._add_elem_to_list(self.get(FILE_HISTORY, []), (file_path, load_method)))
+        self.set(FILE_HISTORY, self._add_elem_to_list(self.get(FILE_HISTORY, []), [file_path, load_method]))
         self.add_path_history(os.path.dirname(file_path[0]))
 
     def set(self, key_path: str, value):
@@ -551,13 +602,12 @@ class BaseSettings(ViewSettings):
     def load_part(self, file_path):
         data = self.load_metadata(file_path)
         bad_key = []
-        if isinstance(data, dict):
-            if not check_loaded_dict(data):
-                for k, v in data.items():
-                    if not check_loaded_dict(v):
-                        bad_key.append(k)
-                for el in bad_key:
-                    del data[el]
+        if isinstance(data, dict) and not check_loaded_dict(data):
+            for k, v in data.items():
+                if not check_loaded_dict(v):
+                    bad_key.append(k)
+            for el in bad_key:
+                del data[el]
         elif isinstance(data, ProfileDict) and not data.verify_data():
             bad_key = data.filter_data()
         return data, bad_key
