@@ -1,6 +1,5 @@
 import collections
 import typing
-from abc import ABCMeta, abstractmethod
 from copy import deepcopy
 from enum import Enum
 
@@ -19,16 +18,15 @@ from qtpy.QtWidgets import (
     QVBoxLayout,
     QWidget,
 )
-from six import with_metaclass
 
 from PartSeg.common_gui.error_report import ErrorDialog
 from PartSegCore.algorithm_describe_base import AlgorithmDescribeBase, AlgorithmProperty, ROIExtractionProfile
 from PartSegCore.channel_class import Channel
 from PartSegCore.image_operations import RadiusType
 from PartSegCore.segmentation.algorithm_base import (
+    ROIExtractionResult,
     SegmentationAlgorithm,
     SegmentationLimitException,
-    SegmentationResult,
 )
 from PartSegImage import Image
 
@@ -39,14 +37,22 @@ from .universal_gui_part import ChannelComboBox, CustomDoubleSpinBox, CustomSpin
 
 
 def update(d, u):
+    if not isinstance(d, dict):
+        d = {}
     for k, v in u.items():
-        if isinstance(v, collections.abc.Mapping):
-            d[k] = update(d.get(k, {}), v)
-        else:
-            d[k] = v
+        d[k] = update(d.get(k, {}), v) if isinstance(v, collections.abc.Mapping) else v
     return d
 
 
+def _pretty_print(data, indent=2) -> str:
+    if isinstance(data, dict):
+        res = "\n"
+        for k, v in data.items():
+            res += f"{' ' * indent}{k}: {_pretty_print(v, indent+2)}\n"
+        return res[:-1]
+    return str(data)
+
+
 class QtAlgorithmProperty(AlgorithmProperty):
     qt_class_dict = {
         int: CustomSpinBox,
@@ -72,8 +78,11 @@ class QtAlgorithmProperty(AlgorithmProperty):
         return self.get_value()
 
     def set_value(self, val):
-        """set value of widget """
-        return self._setter(self._widget, val)
+        """set value of widget"""
+        try:
+            return self._setter(self._widget, val)
+        except TypeError:
+            pass
 
     def get_field(self) -> QWidget:
         """
@@ -99,7 +108,7 @@ class QtAlgorithmProperty(AlgorithmProperty):
                 default_value=ob.default_value,
                 options_range=ob.range,
                 single_steep=ob.single_step,
-                property_type=ob.value_type,
+                value_type=ob.value_type,
                 possible_values=ob.possible_values,
                 help_text=ob.help_text,
                 per_dimension=ob.per_dimension,
@@ -149,12 +158,14 @@ class QtAlgorithmProperty(AlgorithmProperty):
         elif issubclass(self.value_type, list):
             res = QComboBox()
             res.addItems(list(map(str, self.possible_values)))
+        elif hasattr(self.value_type, "get_object"):
+            res = self.value_type.get_object()
         else:
             raise ValueError(f"Unknown class: {self.value_type}")
         tool_tip_text = ""
         if self.help_text:
             tool_tip_text = self.help_text
-        tool_tip_text += f" default value: {str(self.default_value)}"
+        tool_tip_text += f" default value: {_pretty_print(self.default_value)}"
         res.setToolTip(tool_tip_text)
         return res
 
@@ -172,6 +183,8 @@ class QtAlgorithmProperty(AlgorithmProperty):
             return widget.values_changed
         if isinstance(widget, ListInput):
             return widget.change_signal
+        if hasattr(widget, "values_changed"):
+            return widget.values_changed
         raise ValueError(f"Unsupported type: {type(widget)}")
 
     @staticmethod
@@ -209,6 +222,8 @@ class QtAlgorithmProperty(AlgorithmProperty):
             return widget.__class__.get_values, widget.__class__.set_values
         if isinstance(widget, ListInput):
             return widget.__class__.get_value, widget.__class__.set_value
+        if hasattr(widget, "get_value") and hasattr(widget, "set_value"):
+            return widget.__class__.get_value, widget.__class__.set_value
         raise ValueError(f"Unsupported type: {type(widget)}")
 
 
@@ -419,18 +434,6 @@ class SubAlgorithmWidget(QWidget):
             painter.drawRect(event.rect())
 
 
-class AbstractAlgorithmSettingsWidget(with_metaclass(ABCMeta, object)):
-    def __init__(self):
-        pass
-
-    @abstractmethod
-    def get_values(self):
-        """
-        :return: dict[str, object]
-        """
-        return dict()
-
-
 class BaseAlgorithmSettingsWidget(QScrollArea):
     values_changed = Signal()
     algorithm_thread: SegmentationThread
@@ -564,7 +567,7 @@ class InteractiveAlgorithmSettingsWidget(BaseAlgorithmSettingsWidget):
 class AlgorithmChoose(QWidget):
     finished = Signal()
     started = Signal()
-    result = Signal(SegmentationResult)
+    result = Signal(ROIExtractionResult)
     value_changed = Signal()
     progress_signal = Signal(str, int)
     algorithm_changed = Signal(str)
@@ -630,9 +633,8 @@ class AlgorithmChoose(QWidget):
         )
 
     def recursive_get_values(self):
-        result = {}
-        for key, widget in self.algorithm_dict.items():
-            result[key] = widget.recursive_get_values()
+        result = {key: widget.recursive_get_values() for key, widget in self.algorithm_dict.items()}
+
         self.settings.set("algorithm_widget_state", update(self.settings.get("algorithm_widget_state", dict), result))
         return result
 
@@ -657,9 +659,9 @@ class AlgorithmChoose(QWidget):
 
     def image_changed(self):
         current_widget: InteractiveAlgorithmSettingsWidget = self.stack_layout.currentWidget()
+        current_widget.image_changed(self.settings.image)
         if hasattr(self.settings, "mask") and hasattr(current_widget, "change_mask"):
             current_widget.change_mask()
-        current_widget.image_changed(self.settings.image)
 
     def mask_changed(self):
         current_widget: InteractiveAlgorithmSettingsWidget = self.stack_layout.currentWidget()
