@@ -1,4 +1,5 @@
 import itertools
+import logging
 from dataclasses import dataclass, field
 from enum import Enum
 from functools import partial
@@ -6,24 +7,27 @@ from typing import Dict, List, Optional, Tuple, Union
 
 import numpy as np
 
+from PartSegCore.class_generator import enum_register
+
 try:
-    from napari._qt.widgets.qt_viewer_buttons import QtViewerPushButton
-except ImportError:
     from napari._qt.qt_viewer_buttons import QtViewerPushButton
+except ImportError:
+    from napari._qt.widgets.qt_viewer_buttons import QtViewerPushButton
+
 
 from napari.components import ViewerModel as Viewer
-from napari.layers import Layer
+from napari.layers import Layer, Points
 from napari.layers.image import Image as NapariImage
 from napari.layers.image._image_constants import Interpolation3D
 from napari.layers.labels import Labels
-from napari.qt import QtNDisplayButton, QtViewer
+from napari.qt import QtViewer
 from napari.qt.threading import thread_worker
 from qtpy.QtCore import QEvent, QObject, QPoint, Qt, Signal
 from qtpy.QtWidgets import QCheckBox, QHBoxLayout, QLabel, QMenu, QToolTip, QVBoxLayout, QWidget
 from vispy.color import Color, ColorArray, Colormap
 from vispy.scene import BaseCamera
 
-from PartSegCore.color_image import ColorMap, calculate_borders, create_color_map
+from PartSegCore.color_image import calculate_borders
 from PartSegCore.image_operations import NoiseFilterType, gaussian, median
 from PartSegCore.roi_info import ROIInfo
 from PartSegImage import Image
@@ -31,6 +35,27 @@ from PartSegImage import Image
 from ..common_backend.base_settings import BaseSettings, ViewSettings
 from .channel_control import ChannelProperty, ColorComboBoxGroup
 
+try:
+    from napari.qt import QtStateButton
+
+    class QtNDisplayButton(QtStateButton):
+        def __init__(self, viewer):
+            super().__init__(
+                "ndisplay_button",
+                viewer.dims,
+                "ndisplay",
+                viewer.dims.events.ndisplay,
+                2,
+                3,
+            )
+
+
+except ImportError:
+    from napari.qt import QtNDisplayButton
+
+    # FIXME when bump minimal version of napari to 0.4.7
+
+
 ORDER_DICT = {"xy": [0, 1, 2, 3], "zy": [0, 2, 1, 3], "zx": [0, 3, 1, 2]}
 NEXT_ORDER = {"xy": "zy", "zy": "zx", "zx": "xy"}
 
@@ -57,7 +82,7 @@ class ImageInfo:
         if not self.layers:
             return np.array(coords)
         fst_layer = self.layers[0]
-        return np.subtract(coords, fst_layer.translate_grid).astype(np.int)
+        return np.subtract(coords, fst_layer.translate_grid).astype(int)
 
 
 class LabelEnum(Enum):
@@ -66,7 +91,7 @@ class LabelEnum(Enum):
     Show_selected = 2
 
     def __str__(self):
-        if self.value == 0:
+        if self.value == 0:  # pylint: disable=W0143
             return "Don't show"
         return self.name.replace("_", " ")
 
@@ -149,7 +174,7 @@ class ImageView(QWidget):
         parent: Optional[QWidget] = None,
         ndisplay=2,
     ):
-        super(ImageView, self).__init__(parent=parent)
+        super().__init__(parent=parent)
 
         self.settings = settings
         self.channel_property = channel_property
@@ -159,6 +184,7 @@ class ImageView(QWidget):
         self._current_order = "xy"
         self.components = None
         self.worker_list = []
+        self.points_layer = None
 
         self.viewer = Viewer(ndisplay=ndisplay)
         self.viewer.theme = self.settings.theme_name
@@ -167,6 +193,9 @@ class ImageView(QWidget):
         self.channel_control = ColorComboBoxGroup(settings, name, channel_property, height=30)
         self.ndim_btn = QtNDisplayButton(self.viewer)
         self.reset_view_button = QtViewerPushButton(self.viewer, "home", "Reset view", self._reset_view)
+        self.points_view_button = QtViewerPushButton(
+            self.viewer, "new_points", "Show points", self.toggle_points_visibility
+        )
         self.roll_dim_button = QtViewerPushButton(self.viewer, "roll", "Roll dimension", self._rotate_dim)
         self.roll_dim_button.setContextMenuPolicy(Qt.CustomContextMenu)
         self.roll_dim_button.customContextMenuRequested.connect(self._dim_order_menu)
@@ -177,6 +206,7 @@ class ImageView(QWidget):
         self.btn_layout.addWidget(self.reset_view_button)
         self.btn_layout.addWidget(self.ndim_btn)
         self.btn_layout.addWidget(self.roll_dim_button)
+        self.btn_layout.addWidget(self.points_view_button)
         self.btn_layout.addWidget(self.channel_control, 1)
         self.btn_layout.addWidget(self.mask_label)
         self.btn_layout.addWidget(self.mask_chk)
@@ -197,6 +227,7 @@ class ImageView(QWidget):
         settings.roi_clean.connect(self.set_roi)
         settings.image_changed.connect(self.set_image)
         settings.image_spacing_changed.connect(self.update_spacing_info)
+        settings.points_changed.connect(self.update_points)
         # settings.labels_changed.connect(self.paint_layer)
         self.old_scene: BaseCamera = self.viewer_widget.view.scene
 
@@ -210,14 +241,14 @@ class ImageView(QWidget):
         except AttributeError:
             self.viewer.dims.events.axis.connect(self._view_changed, position="last")
         self.viewer.dims.events.ndisplay.connect(self._view_changed, position="last")
-        if hasattr(self.viewer.dims.events, "ndisplay"):
-            self.viewer.dims.events.ndisplay.connect(self._view_changed, position="last")
-            self.viewer.dims.events.ndisplay.connect(self.camera_change, position="last")
-        else:
-            self.viewer.dims.events.camera.connect(self._view_changed, position="last")
-            self.viewer.dims.events.camera.connect(self.camera_change, position="last")
+        self.viewer.dims.events.ndisplay.connect(self._view_changed, position="last")
+        self.viewer.dims.events.ndisplay.connect(self.camera_change, position="last")
         self.viewer.events.reset_view.connect(self._view_changed, position="last")
 
+    def toggle_points_visibility(self):
+        if self.points_layer is not None:
+            self.points_layer.visible = not self.points_layer.visible
+
     def _dim_order_menu(self, point: QPoint):
         menu = QMenu()
         for key in ORDER_DICT:
@@ -300,10 +331,27 @@ class ImageView(QWidget):
         if image_info.mask is not None:
             image_info.mask.scale = image.normalized_scaling()
 
+    def _active_layer(self):
+        if hasattr(self.viewer.layers, "selection"):
+            return self.viewer.layers.selection.active
+        return self.viewer.active_layer
+
+    def _coordinates(self):
+        active_layer = self._active_layer()
+        if active_layer is None:
+            return
+        if (
+            hasattr(self.viewer, "cursor")
+            and hasattr(self.viewer.cursor, "position")
+            and hasattr(active_layer, "world_to_data")
+        ):
+            return [int(x) for x in active_layer.world_to_data(self.viewer.cursor.position)]
+        return [int(x) for x in active_layer.coordinates]
+
     def print_info(self, value):
-        if not self.viewer.active_layer:
+        cords = self._coordinates()
+        if cords is None:
             return
-        cords = np.array([int(x) for x in self.viewer.active_layer.coordinates])
         bright_array = []
         components = []
         for image_info in self.image_info.values():
@@ -323,10 +371,7 @@ class ImageView(QWidget):
             return
         text = f"{cords}: "
         if bright_array:
-            if len(bright_array) == 1:
-                text += str(bright_array[0])
-            else:
-                text += str(bright_array)
+            text += str(bright_array[0]) if len(bright_array) == 1 else str(bright_array)
         self.components = components
         if components:
             if len(components) == 1:
@@ -338,10 +383,6 @@ class ImageView(QWidget):
     def get_control_view(self) -> ImageShowState:
         return self.image_state
 
-    @staticmethod
-    def convert_to_vispy_colormap(colormap: ColorMap):
-        return Colormap(ColorArray(create_color_map(colormap) / 255))
-
     def mask_opacity(self) -> float:
         """Get mask opacity"""
         return self.settings.get_from_profile("mask_presentation_opacity", 1)
@@ -358,12 +399,25 @@ class ImageView(QWidget):
             return self.settings.image
         return self.image_info[self.current_image].image
 
+    def update_points(self):
+        if self.settings.points is not None:
+            self.points_view_button.setVisible(True)
+            if self.points_layer is None or self.points_layer not in self.viewer.layers:
+                self.points_layer = Points(self.settings.points, scale=self.settings.image.normalized_scaling())
+                self.viewer.add_layer(self.points_layer)
+            else:
+                self.points_layer.data = self.settings.points
+                self.points_layer.scale = self.settings.image.normalized_scaling()
+        elif self.points_layer is not None and self.points_layer in self.viewer.layers:
+            self.points_view_button.setVisible(False)
+            self.points_layer.data = np.empty((0, 4))
+
     def set_roi(self, roi_info: Optional[ROIInfo] = None, image: Optional[Image] = None) -> None:
         image = self.get_image(image)
         if roi_info is None:
             roi_info = self.settings.roi_info
         image_info = self.image_info[image.file_path]
-        if image_info.roi is not None:
+        if image_info.roi is not None and image_info.roi in self.viewer.layers:
             self.viewer.layers.unselect_all()
             image_info.roi.selected = True
             self.viewer.layers.remove_selected()
@@ -403,11 +457,17 @@ class ImageView(QWidget):
             image_info.roi.opacity = self.image_state.opacity
 
     def remove_all_roi(self):
-        self.viewer.layers.unselect_all()
+        if hasattr(self.viewer.layers, "selection"):
+            self.viewer.layers.selection.clear()
+        else:
+            self.viewer.layers.unselect_all()
         for image_info in self.image_info.values():
             if image_info.roi is None:
                 continue
-            image_info.roi.selected = True
+            if hasattr(self.viewer.layers, "selection"):
+                self.viewer.layers.selection.add(image_info.roi)
+            else:
+                image_info.roi.selected = True
             image_info.roi = None
 
         self.viewer.layers.remove_selected()
@@ -500,8 +560,7 @@ class ImageView(QWidget):
 
     def _remove_worker(self, sender):
         for worker in self.worker_list:
-            signals = "_signals" if hasattr(worker, "_signals") else "signals"
-            if sender is getattr(worker, signals):
+            if sender is worker.signals:
                 self.worker_list.remove(worker)
                 break
         else:
@@ -539,14 +598,22 @@ class ImageView(QWidget):
 
         filters = self.channel_control.get_filter()
         for i, layer in enumerate(image_info.layers):
-            self._add_layer_util(i, layer, filters)
+            try:
+                self._add_layer_util(i, layer, filters)
+            except AssertionError:
+                layer.colormap = "gray"
+                self._add_layer_util(i, layer, filters)
 
         self.image_info[image.file_path].filter_info = filters
         self.image_info[image.file_path].layers = image_info.layers
         self.current_image = image.file_path
         self.viewer.reset_view()
         if self.viewer.layers:
-            self.viewer.layers[-1].selected = True
+            if hasattr(self.viewer.layers, "selection"):
+                self.viewer.layers.selection.clear()
+                self.viewer.layers.selection.add(self.viewer.layers[-1])
+            else:
+                self.viewer.layers[-1].selected = True
 
         for i, axis in enumerate(image.axis_order):
             if axis == "C":
@@ -572,7 +639,7 @@ class ImageView(QWidget):
 
         channels = image.channels
         if self.image_info and not replace:
-            channels = max(channels, *[x.image.channels for x in self.image_info.values()])
+            channels = max(channels, *(x.image.channels for x in self.image_info.values()))
 
         self.channel_control.set_channels(channels)
         visibility = self.channel_control.channel_visibility
@@ -580,9 +647,7 @@ class ImageView(QWidget):
         ranges = image.get_ranges()
         limits = [ranges[i] if x is None else x for i, x in zip(range(image.channels), limits)]
         gamma = self.channel_control.get_gamma()
-        colormaps = [
-            self.convert_to_vispy_colormap(self.channel_control.selected_colormaps[i]) for i in range(image.channels)
-        ]
+        colormaps = [self.channel_control.selected_colormaps[i] for i in range(image.channels)]
         parameters = ImageParameters(
             limits, visibility, gamma, colormaps, image.normalized_scaling(), len(self.viewer.layers)
         )
@@ -642,9 +707,7 @@ class ImageView(QWidget):
             if len(image_info.layers) > index:
                 image_info.layers[index].visible = self.channel_control.channel_visibility[index]
                 if self.channel_control.channel_visibility[index]:
-                    image_info.layers[index].colormap = self.convert_to_vispy_colormap(
-                        self.channel_control.selected_colormaps[index]
-                    )
+                    image_info.layers[index].colormap = self.channel_control.selected_colormaps[index]
                     limits = self.channel_control.get_limits()[index]
                     limits = image_info.image.get_ranges()[index] if limits is None else limits
                     image_info.layers[index].contrast_limits = limits
@@ -665,14 +728,15 @@ class ImageView(QWidget):
     def closeEvent(self, event):
         for worker in self.worker_list:
             worker.quit()
+        self.viewer.layers.clear()
+        self.viewer_widget.close()
         super().closeEvent(event)
 
     def get_tool_tip_text(self) -> str:
         image = self.settings.image
         image_info = self.image_info[image.file_path]
-        text_list = []
-        for el in self.components:
-            text_list.append(_print_dict(image_info.roi_info.annotations.get(el, {})))
+        text_list = [_print_dict(image_info.roi_info.annotations.get(el, {})) for el in self.components]
+
         return " ".join(text_list)
 
     def event(self, event: QEvent):
@@ -690,6 +754,17 @@ class NapariQtViewer(QtViewer):
         """
         event.ignore()
 
+    def close(self):
+        self.dockConsole.deleteLater()
+        self.dockLayerList.deleteLater()
+        self.dockLayerControls.deleteLater()
+        self.activityDock.deleteLater()
+        return super().close()
+
+    def closeEvent(self, event):
+        self.close()
+        super().closeEvent(event)
+
 
 @dataclass
 class ImageParameters:
@@ -728,6 +803,9 @@ prepare_layers = thread_worker(_prepare_layers)
 
 
 def _print_dict(dkt: dict, indent=""):
+    if not isinstance(dkt, dict):
+        logging.error(f"{type(dkt)} instead of dict passed to _print_dict")
+        return indent + str(dkt)
     res = []
     for k, v in dkt.items():
         if isinstance(v, dict):
@@ -735,3 +813,6 @@ def _print_dict(dkt: dict, indent=""):
         else:
             res.append(f"{indent}{k}: {v}")
     return "\n".join(res)
+
+
+enum_register.register_class(LabelEnum)
