@@ -29,7 +29,7 @@ from ..common_backend.base_settings import BaseSettings
 
 class AcceptFiles(QDialog):
     def __init__(self, files):
-        super(AcceptFiles, self).__init__()
+        super().__init__()
         self.ok = QPushButton("Add", self)
         self.ok.clicked.connect(self.accept)
         discard = QPushButton("Discard", self)
@@ -44,7 +44,7 @@ class AcceptFiles(QDialog):
         self.ok.setAutoDefault(True)
 
         layout = QVBoxLayout()
-        layout.addWidget(QLabel("Found {} files".format(len(files))))
+        layout.addWidget(QLabel(f"Found {len(files)} files"))
         layout.addWidget(self.files)
         butt_layout = QHBoxLayout()
         butt_layout.addWidget(discard)
@@ -67,7 +67,7 @@ class FileListItem(QListWidgetItem):
     def __init__(self, file_path):
         size = os.stat(file_path).st_size
         size = float(size) / (1024 ** 2)
-        super().__init__("{:s} ({:.2f} MB)".format(file_path, size))
+        super().__init__(f"{file_path:s} ({size:.2f} MB)")
         self.setTextAlignment(Qt.AlignRight)
         self.file_path = file_path
 
@@ -79,12 +79,12 @@ class FileListWidget(QListWidget):
 
 
 class AddFiles(QWidget):
-    """Docstring for AddFiles. """
+    """Docstring for AddFiles."""
 
     file_list_changed = Signal(set)
 
     def __init__(self, settings: BaseSettings, parent=None, btn_layout=QHBoxLayout):
-        """TODO: to be defined1. """
+        """TODO: to be defined1."""
         QWidget.__init__(self, parent)
         self.mask_list: List[MaskMapper] = []
         self.settings = settings
@@ -171,7 +171,7 @@ class AddFiles(QWidget):
 
     def find_all(self):
         paths = glob(str(self.paths_input.text()))
-        paths = sorted([x for x in (set(paths) - self.files_to_proceed) if not os.path.isdir(x)])
+        paths = sorted(x for x in (set(paths) - self.files_to_proceed) if not os.path.isdir(x))
         if len(paths) > 0:
             self.update_files_list(paths)
 
