@@ -1,29 +1,25 @@
-from __future__ import division, print_function
-
 import os
 from math import log
 from typing import List, Union
 
 import numpy as np
+from napari.utils import Colormap
+from napari.utils.colormaps import make_colorbar
 from qtpy import QtGui
 from qtpy.QtCore import QRect, QSize
 from qtpy.QtGui import QIcon, QPainter
 from qtpy.QtWidgets import QLabel, QToolButton
 
 from PartSeg.common_gui.numpy_qimage import NumpyQImage
-from PartSegCore.class_generator import enum_register
-from PartSegCore.color_image.color_image_base import color_bar_fun
 from PartSegData import icons_dir
 
 from ..common_backend.base_settings import ViewSettings
-from .napari_image_view import ImageView, LabelEnum
+from .napari_image_view import ImageView
 
 canvas_icon_size = QSize(20, 20)
 step = 1.01
 max_step = log(1.2, step)
 
-enum_register.register_class(LabelEnum)
-
 
 def create_tool_button(text, icon):
     res = QToolButton()
@@ -74,9 +70,10 @@ class ColorBar(QLabel):
             self.round_range = self.round_range[0], self.round_range[1] - round_factor
         # print(self.range, self.round_range)
         data = np.linspace(0, 1, 512)
-        data = (data ** gamma) * 255
-        img = color_bar_fun(data.reshape((1, 512))[:, ::-1], cmap)
-        self.image = NumpyQImage(np.swapaxes(img, 0, 1))
+        interpolated = cmap.map(data)
+        data = data ** gamma
+        colormap = Colormap(interpolated, controls=data)
+        self.image = NumpyQImage(np.array(make_colorbar(colormap, size=(512, 1), horizontal=False)[::-1]))
         self.repaint()
 
     @staticmethod
