@@ -2,14 +2,10 @@ import argparse
 import logging
 import multiprocessing
 import os
+import platform
 import sys
 from functools import partial
 
-try:
-    from napari._qt.qthreading import wait_for_workers_to_quit
-except ImportError:
-    from napari._qt.threading import wait_for_workers_to_quit
-
 from qtpy.QtCore import Qt
 from qtpy.QtGui import QFontDatabase
 
@@ -28,7 +24,13 @@ def _test_imports():
     from qtpy.QtWidgets import QApplication
 
     app = QApplication([])
-    from napari._qt.widgets.qt_console import QtConsole
+    import napari
+    from packaging.version import parse
+
+    if parse(napari.__version__) < parse("0.4.5"):
+        from napari._qt.widgets.qt_console import QtConsole
+    else:
+        from napari_console.qt_console import QtConsole
 
     from PartSeg import plugins
     from PartSeg._launcher.main_window import MainWindow
@@ -41,7 +43,10 @@ def _test_imports():
     w1 = AnalysisMain("test")
     w2 = MaskMain("test")
     w3 = MainWindow("test")
-    console = QtConsole()
+    if parse(napari.__version__) < parse("0.4.5"):
+        console = QtConsole()
+    else:
+        console = QtConsole(napari.Viewer())
     if QFontDatabase.addApplicationFont(os.path.join(font_dir, "Symbola.ttf")) == -1:
         raise ValueError("Error with loading Symbola font")
     del w1
@@ -75,10 +80,18 @@ def main():
     # print(args)
 
     logging.basicConfig(level=logging.INFO)
+    if platform.system() == "Darwin":
+        multiprocessing.set_start_method("spawn")
+
     CustomApplication.setAttribute(Qt.AA_EnableHighDpiScaling)
     my_app = CustomApplication(sys.argv, name="PartSeg", icon=os.path.join(icons_dir, "icon.png"))
     my_app.check_release()
-    my_app.aboutToQuit.connect(wait_for_workers_to_quit)
+    try:
+        from napari.qt import get_app
+
+        get_app()
+    except ImportError:
+        pass
     QFontDatabase.addApplicationFont(os.path.join(font_dir, "Symbola.ttf"))
     if args.gui == "roi_analysis" or args.mf:
         from PartSeg import plugins
@@ -110,6 +123,11 @@ def main():
         title = f"{APP_NAME} Launcher"
         wind = MainWindow(title=title)
 
+    try:
+        from napari._qt.qthreading import wait_for_workers_to_quit
+    except ImportError:
+        from napari._qt.threading import wait_for_workers_to_quit
+    my_app.aboutToQuit.connect(wait_for_workers_to_quit)
     wind.show()
     rc = my_app.exec_()
     del wind
