@@ -22,6 +22,7 @@ Calculation hierarchy:
 """
 import json
 import logging
+import os
 import threading
 import traceback
 import uuid
@@ -63,10 +64,11 @@ from PartSegCore.analysis.measurement_base import AreaType, PerComponent
 from PartSegCore.analysis.measurement_calculation import MeasurementResult
 from PartSegCore.analysis.save_functions import save_dict
 from PartSegCore.mask_create import calculate_mask
-from PartSegCore.segmentation.algorithm_base import AdditionalLayerDescription, SegmentationAlgorithm, report_empty_fun
+from PartSegCore.segmentation.algorithm_base import SegmentationAlgorithm, report_empty_fun
 from PartSegImage import Image, TiffImageReader
 
-from ...io_utils import HistoryElement, WrongFileTypeException
+from ...io_utils import WrongFileTypeException
+from ...project_info import AdditionalLayerDescription, HistoryElement
 from ...roi_info import ROIInfo
 from ...segmentation import RestartableAlgorithm
 from .. import PartEncoder
@@ -122,7 +124,7 @@ class CalculationProcess:
 
     def __init__(self):
         self.reused_mask = set()
-        self.mask_dict = dict()
+        self.mask_dict = {}
         self.calculation = None
         self.measurement: List[MeasurementResult] = []
         self.image: Optional[Image] = None
@@ -176,7 +178,7 @@ class CalculationProcess:
             if operation == RootType.Project:
                 self.mask = project.mask
                 # FIXME when load annotation from project is done
-                self.roi_info = ROIInfo(project.roi)
+                self.roi_info = project.roi_info
                 self.additional_layers = project.additional_layers
                 self.history = project.history
                 self.algorithm_parameters = project.algorithm_parameters
@@ -214,6 +216,8 @@ class CalculationProcess:
         mask_path = operation.get_mask_path(self.calculation.file_path)
         if mask_path == "":  # pragma: no cover
             raise ValueError("Empty path to mask.")
+        if not os.path.exists(mask_path):
+            raise OSError(f"Mask file {mask_path} does not exists")
         with tifffile.TiffFile(mask_path) as mask_file:
             mask = mask_file.asarray()
             mask = TiffImageReader.update_array_shape(mask, mask_file.series[0].axes)[..., 0]
@@ -292,7 +296,7 @@ class CalculationProcess:
         project_tuple = ProjectTuple(
             file_path="",
             image=self.image,
-            roi=self.roi_info.roi,  # FIXME
+            roi_info=self.roi_info,
             additional_layers=self.additional_layers,
             mask=self.mask,
             history=self.history,
@@ -310,7 +314,7 @@ class CalculationProcess:
         """
         mask = calculate_mask(
             mask_description=operation.mask_property,
-            segmentation=self.roi_info.roi,
+            roi=self.roi_info.roi,
             old_mask=self.mask,
             spacing=self.image.spacing,
             time_axis=self.image.time_pos,
@@ -380,7 +384,7 @@ class CalculationProcess:
         elif isinstance(node.operation, MeasurementCalculate):
             self.step_measurement(node.operation)
         else:  # pragma: no cover
-            raise ValueError("Unknown operation {} {}".format(type(node.operation), node.operation))
+            raise ValueError(f"Unknown operation {type(node.operation)} {node.operation}")
 
 
 class BatchResultDescription(NamedTuple):
@@ -458,7 +462,7 @@ class CalculationManager:
 
         :param int val: number of workers.
         """
-        logging.debug("Number off process {}".format(val))
+        logging.debug(f"Number off process {val}")
         self.batch_manager.set_number_of_process(val)
 
     def get_results(self) -> BatchResultDescription:
@@ -480,10 +484,7 @@ class CalculationManager:
                     for err in errors:
                         new_errors.append((el.path_to_file, err))
                 else:
-                    if ind != -1:
-                        file_info = calculation.file_list[ind]
-                    else:
-                        file_info = "unknown file"
+                    file_info = calculation.file_list[ind] if ind != -1 else "unknown file"
                     self.writer.add_calculation_error(calculation, file_info, el[0])
                     self.errors_list.append((file_info, el))
                     new_errors.append((file_info, el))
@@ -508,13 +509,23 @@ class SheetData:
     Store single sheet information
     """
 
-    def __init__(self, name: str, columns: List[Tuple[str, str]]):
+    def __init__(self, name: str, columns: List[Tuple[str, str]], raw=False):
+        if len(columns) != len(set(columns)):
+            raise ValueError(f"Columns should be unique: {columns}")
         self.name = name
-        self.columns = pd.MultiIndex.from_tuples([("name", "units")] + columns)
+        if raw:
+            self.columns = pd.MultiIndex.from_tuples(columns)
+        else:
+            self.columns = pd.MultiIndex.from_tuples([("name", "units")] + columns)
         self.data_frame = pd.DataFrame([], columns=self.columns)
         self.row_list: List[Any] = []
 
     def add_data(self, data, ind):
+        if len(data) != len(self.columns):
+            raise ValueError(
+                f"Wrong number of columns in data ({len(data)} instead of "
+                f"{len(self.columns)} {data} for columns {self.columns.values}"
+            )
         if ind is None:
             ind = len(self.row_list)
         self.row_list.append((ind, data))
@@ -522,7 +533,8 @@ class SheetData:
     def add_data_list(self, data, ind):
         if ind is None:
             ind = len(self.row_list)
-        self.row_list.extend([(ind, x) for x in data])
+        for x in data:
+            self.add_data(x, ind)
 
     def get_data_to_write(self) -> Tuple[str, pd.DataFrame]:
         """
@@ -567,7 +579,7 @@ class FileData:
         else:  # pragma: no cover
             self.file_type = FileType.text_file
         self.writing = False
-        data = SheetData("calculation_info", [("Description", "str"), ("JSON", "str")])
+        data = SheetData("calculation_info", [("Description", "str"), ("JSON", "str")], raw=True)
         data.add_data([str(calculation.calculation_plan), json.dumps(calculation.calculation_plan, cls=PartEncoder)], 0)
         self.sheet_dict = {}
         self.calculation_info = {}
@@ -601,9 +613,9 @@ class FileData:
         if self.file_type == FileType.text_file:
             return False, "Text file allow store only one sheet"
         if self.component_str in name:
-            return False, "Sequence '{}' is reserved for auto generated sheets".format(FileData.component_str)
+            return False, f"Sequence '{FileData.component_str}' is reserved for auto generated sheets"
         if name in self.sheet_set:
-            return False, "Sheet name {} already in use".format(name)
+            return False, f"Sheet name {name} already in use"
         return True, ""
 
     def add_data_part(self, calculation: BaseCalculation):
@@ -615,11 +627,9 @@ class FileData:
             or :py:attr:`sheet_name` name already is in use.
         """
         if calculation.measurement_file_path != self.file_path:
-            raise ValueError(
-                "[FileData] different file path {} vs {}".format(calculation.measurement_file_path, self.file_path)
-            )
+            raise ValueError(f"[FileData] different file path {calculation.measurement_file_path} vs {self.file_path}")
         if calculation.sheet_name in self.sheet_set:  # pragma: no cover
-            raise ValueError("[FileData] sheet name {} already in use".format(calculation.sheet_name))
+            raise ValueError(f"[FileData] sheet name {calculation.sheet_name} already in use")
         measurement = calculation.calculation_plan.get_measurements()
         component_information = [x.measurement_profile.get_component_info(x.units) for x in measurement]
         num = 1
@@ -726,7 +736,7 @@ class FileData:
                     try:
                         self.write_to_excel(file_path, data)
                         break
-                    except (PermissionError, IOError):
+                    except (PermissionError, OSError):
                         base, ext = path.splitext(self.file_path)
                         file_path = f"{base}({i}){ext}"
                 if i == 100:  # pragma: no cover
