@@ -6,10 +6,11 @@ import numpy as np
 from PartSegCore.algorithm_describe_base import ROIExtractionProfile
 from PartSegCore.analysis.algorithm_description import analysis_algorithm_dict
 from PartSegCore.analysis.analysis_utils import SegmentationPipeline
-from PartSegCore.io_utils import HistoryElement
 from PartSegCore.mask_create import calculate_mask
+from PartSegCore.project_info import HistoryElement
+from PartSegCore.roi_info import ROIInfo
 from PartSegCore.segmentation import RestartableAlgorithm
-from PartSegCore.segmentation.algorithm_base import AdditionalLayerDescription
+from PartSegCore.segmentation.algorithm_base import AdditionalLayerDescription, ROIExtractionResult
 from PartSegImage import Image
 
 
@@ -19,7 +20,7 @@ def _empty_fun(_a1, _a2):
 
 @dataclass(frozen=True)
 class PipelineResult:
-    roi: np.ndarray
+    roi_info: ROIInfo
     additional_layers: typing.Dict[str, AdditionalLayerDescription]
     mask: np.ndarray
     history: typing.List[HistoryElement]
@@ -31,25 +32,27 @@ def calculate_pipeline(image: Image, mask: typing.Optional[np.ndarray], pipeline
     report_fun("max", 2 * len(pipeline.mask_history) + 1)
     for i, el in enumerate(pipeline.mask_history):
         result, _ = calculate_segmentation_step(el.segmentation, image, mask)
-        segmentation = result.roi
+        roi_info = result.roi_info.fit_to_image(image)
         report_fun("step", 2 * i + 1)
         new_mask = calculate_mask(
             mask_description=el.mask_property,
-            segmentation=segmentation,
+            roi=roi_info.roi,
             old_mask=mask,
             spacing=image.spacing,
             time_axis=image.time_pos,
         )
         segmentation_parameters = {"algorithm_name": el.segmentation.name, "values": el.segmentation.values}
-        history.append(HistoryElement.create(segmentation, mask, segmentation_parameters, el.mask_property))
+        history.append(HistoryElement.create(roi_info, mask, segmentation_parameters, el.mask_property))
         report_fun("step", 2 * i + 2)
-        mask = new_mask
+        mask = image.fit_array_to_image(new_mask)
     result, text = calculate_segmentation_step(pipeline.segmentation, image, mask)
     report_fun("step", 2 * len(pipeline.mask_history) + 1)
-    return PipelineResult(result.roi, result.additional_layers, mask, history, text)
+    return PipelineResult(result.roi_info, result.additional_layers, mask, history, text)
 
 
-def calculate_segmentation_step(profile: ROIExtractionProfile, image: Image, mask: typing.Optional[np.ndarray]):
+def calculate_segmentation_step(
+    profile: ROIExtractionProfile, image: Image, mask: typing.Optional[np.ndarray]
+) -> typing.Tuple[ROIExtractionResult, str]:
     algorithm: RestartableAlgorithm = analysis_algorithm_dict[profile.algorithm]()
     algorithm.set_image(image)
     algorithm.set_mask(mask)
