@@ -158,10 +158,8 @@ def get_save_path(op: Save, calculation: "FileCalculation") -> str:
     rel_path = os.path.relpath(calculation.file_path, calculation.base_prefix)
     rel_path = os.path.splitext(rel_path)[0]
     if op.directory:
-        file_path = os.path.join(calculation.result_prefix, rel_path, op.suffix + extension)
-    else:
-        file_path = os.path.join(calculation.result_prefix, rel_path + op.suffix + extension)
-    return file_path
+        return os.path.join(calculation.result_prefix, rel_path, op.suffix + extension)
+    return os.path.join(calculation.result_prefix, rel_path + op.suffix + extension)
 
 
 class MaskMapper:
@@ -282,9 +280,7 @@ class MaskFile(MaskMapper, BaseSerializableClass):
                 try:
                     file_name, mask_name = line.split(sep)
                 except ValueError:
-                    logging.error(
-                        "Error in parsing map file\nline {}\n{}\nfrom file{}".format(i, line, self.path_to_file)
-                    )
+                    logging.error(f"Error in parsing map file\nline {i}\n{line}\nfrom file{self.path_to_file}")
                     continue
                 file_name = file_name.strip()
                 mask_name = mask_name.strip()
@@ -587,10 +583,7 @@ class CalculationPlan:
         for el in node.children:
             if isinstance(el.operation, MaskUse):
                 used_mask.add(el.operation.name)
-            elif isinstance(el.operation, MaskSum):
-                used_mask.add(el.operation.mask1)
-                used_mask.add(el.operation.mask2)
-            elif isinstance(el.operation, MaskIntersection):
+            elif isinstance(el.operation, (MaskSum, MaskIntersection)):
                 used_mask.add(el.operation.mask1)
                 used_mask.add(el.operation.mask2)
         return used_mask
@@ -617,10 +610,9 @@ class CalculationPlan:
             return NodeType.save
         if isinstance(node.operation, MaskUse):
             return NodeType.file_mask
-        if isinstance(node.operation, Operations):
-            if node.operation == Operations.reset_to_base:
-                return NodeType.mask
-        raise ValueError("[get_node_type] unknown node type {}".format(node.operation))
+        if isinstance(node.operation, Operations) and node.operation == Operations.reset_to_base:
+            return NodeType.mask
+        raise ValueError(f"[get_node_type] unknown node type {node.operation}")
 
     def add_step(self, step):
         if self.current_pos is None:
@@ -647,9 +639,7 @@ class CalculationPlan:
 
     def has_children(self):
         node = self.get_node()
-        if len(node.children) > 0:
-            return True
-        return False
+        return len(node.children) > 0
 
     def remove_step(self):
         path = copy(self.current_pos)
@@ -691,11 +681,7 @@ class CalculationPlan:
         """
         :return: list[MaskMapper]
         """
-        mask_mapper_list = []
-        for el in self.execution_tree.children:
-            if isinstance(el.operation, MaskMapper):
-                mask_mapper_list.append(el.operation)
-        return mask_mapper_list
+        return [el.operation for el in self.execution_tree.children if isinstance(el.operation, MaskMapper)]
 
     def set_path_to_mapping_file(self, num, path):
         for el in self.execution_tree.children:
@@ -729,30 +715,29 @@ class CalculationPlan:
         """
         if el.__class__.__name__ not in CalculationPlan.correct_name.keys():
             print(el, el.__class__.__name__, file=sys.stderr)
-            raise ValueError("Unknown type {}".format(el.__class__.__name__))
+            raise ValueError(f"Unknown type {el.__class__.__name__}")
         if isinstance(el, RootType):
             return f"Root: {el}"
-        if isinstance(el, Operations):
-            if el == Operations.reset_to_base:
-                return "reset project to base image with mask"
+        if isinstance(el, Operations) and el == Operations.reset_to_base:
+            return "reset project to base image with mask"
         if isinstance(el, ROIExtractionProfile):
-            return "Segmentation: {}".format(el.name)
+            return f"Segmentation: {el.name}"
         if isinstance(el, MeasurementCalculate):
             if el.name_prefix == "":
-                return "Measurement: {}".format(el.name)
-            return "Measurement: {} with prefix: {}".format(el.name, el.name_prefix)
+                return f"Measurement: {el.name}"
+            return f"Measurement: {el.name} with prefix: {el.name_prefix}"
         if isinstance(el, MaskCreate):
             if el.name != "":
-                return "Create mask: {}".format(el.name)
+                return f"Create mask: {el.name}"
             return "Create mask:"
         if isinstance(el, MaskUse):
-            return "Use mask: {}".format(el.name)
+            return f"Use mask: {el.name}"
         if isinstance(el, MaskSuffix):
-            return "File mask: {} with suffix {}".format(el.name, el.suffix)
+            return f"File mask: {el.name} with suffix {el.suffix}"
         if isinstance(el, MaskSub):
-            return "File mask: {} substitution {} on {}".format(el.name, el.base, el.rep)
+            return f"File mask: {el.name} substitution {el.base} on {el.rep}"
         if isinstance(el, MaskFile):
-            return "File mapping mask: {}, {}".format(el.name, el.path_to_file)
+            return f"File mapping mask: {el.name}, {el.path_to_file}"
         if isinstance(el, Save):
             base = el.short_name
             if el.directory:
@@ -765,14 +750,14 @@ class CalculationPlan:
             return text
         if isinstance(el, MaskIntersection):
             if el.name == "":
-                return "Mask intersection of mask {} and {}".format(el.mask1, el.mask2)
-            return "Mask {} intersection of mask {} and {}".format(el.name, el.mask1, el.mask2)
+                return f"Mask intersection of mask {el.mask1} and {el.mask2}"
+            return f"Mask {el.name} intersection of mask {el.mask1} and {el.mask2}"
         if isinstance(el, MaskSum):
             if el.name == "":
-                return "Mask sum of mask {} and {}".format(el.mask1, el.mask2)
-            return "Mask {} sum of mask {} and {}".format(el.name, el.mask1, el.mask2)
+                return f"Mask sum of mask {el.mask1} and {el.mask2}"
+            return f"Mask {el.name} sum of mask {el.mask1} and {el.mask2}"
 
-        raise ValueError("Unknown type {}".format(type(el)))
+        raise ValueError(f"Unknown type {type(el)}")
 
     def pretty_print(self) -> str:
         return f"Calcualation Plan: {self.name}\n" + self._pretty_print(self.execution_tree, 0)
@@ -791,7 +776,8 @@ class CalculationPlan:
             name += "\n" + textwrap.indent(txt, " " * (indent + 4))
 
         if elem.children:
-            suffix = "\n" + "\n".join([self._pretty_print(x, indent + 2) for x in elem.children])
+            suffix = "\n" + "\n".join(self._pretty_print(x, indent + 2) for x in elem.children)
+
         else:
             suffix = ""
 
