@@ -2,10 +2,25 @@ from collections import OrderedDict
 from enum import Enum
 from functools import reduce
 from math import pi
-from typing import Any, Callable, Dict, Iterator, List, MutableMapping, NamedTuple, Optional, Set, Tuple, Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    Iterator,
+    List,
+    MutableMapping,
+    NamedTuple,
+    Optional,
+    Set,
+    Tuple,
+    Union,
+)
 
 import numpy as np
+import pandas as pd
 import SimpleITK
+from mahotas.features import haralick
 from scipy.spatial.distance import cdist
 from sympy import symbols
 
@@ -22,6 +37,8 @@ from .measurement_base import AreaType, Leaf, MeasurementEntry, MeasurementMetho
 
 # TODO change image to channel in signature of measurement calculate_property
 
+NO_COMPONENT = -1
+
 
 class ProhibitedDivision(Exception):
     pass
@@ -52,7 +69,6 @@ class ComponentsInfo(NamedTuple):
 
 def empty_fun(_a0=None, _a1=None):
     """This function  is be used as dummy reporting function."""
-    pass
 
 
 MeasurementValueType = Union[float, List[float], str]
@@ -71,16 +87,16 @@ class MeasurementResult(MutableMapping[str, MeasurementResultType]):
     def __init__(self, components_info: ComponentsInfo):
         self.components_info = components_info
         self._data_dict = OrderedDict()
-        self._units_dict: Dict[str, str] = dict()
-        self._type_dict: Dict[str, Tuple[PerComponent, AreaType]] = dict()
+        self._units_dict: Dict[str, str] = {}
+        self._type_dict: Dict[str, Tuple[PerComponent, AreaType]] = {}
         self._units_dict["Mask component"] = ""
         self._units_dict["Segmentation component"] = ""
 
     def __str__(self):  # pragma: no cover
-        text = ""
-        for key, val in self._data_dict.items():
-            text += f"{key}: {val}; type {self._type_dict[key]}, units {self._units_dict[key]}\n"
-        return text
+        return "".join(
+            f"{key}: {val}; type {self._type_dict[key]}, units {self._units_dict[key]}\n"
+            for key, val in self._data_dict.items()
+        )
 
     def __setitem__(self, k: str, v: MeasurementResultInputType) -> None:
 
@@ -102,6 +118,14 @@ class MeasurementResult(MutableMapping[str, MeasurementResultType]):
     def __iter__(self) -> Iterator[str]:
         return iter(self._data_dict)
 
+    def to_dataframe(self) -> pd.DataFrame:
+        data = self.get_separated()
+        columns = [
+            f"{label} ({units})" if units else label for label, units in zip(self.get_labels(), self.get_units())
+        ]
+        df = pd.DataFrame(data, columns=columns)
+        return df.astype({"Segmentation component": int}).set_index("Segmentation component")
+
     def set_filename(self, path_fo_file: str):
         """
         Set name of file to be presented as first position.
@@ -155,7 +179,10 @@ class MeasurementResult(MutableMapping[str, MeasurementResultType]):
             name = self._data_dict[FILE_NAME_STR]
             res = [name]
             iterator = iter(self._data_dict.keys())
-            next(iterator)
+            try:
+                next(iterator)
+            except StopIteration:
+                pass
         else:
             res = []
             iterator = iter(self._data_dict.keys())
@@ -169,7 +196,7 @@ class MeasurementResult(MutableMapping[str, MeasurementResultType]):
     def get_separated(self) -> List[List[MeasurementValueType]]:
         """Get measurements separated for each component"""
         has_mask_components, has_segmentation_components = self.get_component_info()
-        if not (has_mask_components or has_segmentation_components):
+        if not has_mask_components and not has_segmentation_components:
             return [list(self._data_dict.values())]
         if has_mask_components and has_segmentation_components:
             translation = self.components_info.components_translation
@@ -185,7 +212,10 @@ class MeasurementResult(MutableMapping[str, MeasurementResultType]):
             name = self._data_dict[FILE_NAME_STR]
             res = [[name] for _ in range(counts)]
             iterator = iter(self._data_dict.keys())
-            next(iterator)
+            try:
+                next(iterator)
+            except StopIteration:
+                pass
         else:
             res = [[] for _ in range(counts)]
             iterator = iter(self._data_dict.keys())
@@ -201,13 +231,12 @@ class MeasurementResult(MutableMapping[str, MeasurementResultType]):
             if per_comp != PerComponent.Yes:
                 for i in range(counts):
                     res[i].append(val)
+            elif area_type == AreaType.ROI:
+                for i, (seg, _mask) in enumerate(component_info):
+                    res[i].append(val[segmentation_to_pos[seg]])
             else:
-                if area_type == AreaType.ROI:
-                    for i, (seg, _mask) in enumerate(component_info):
-                        res[i].append(val[segmentation_to_pos[seg]])
-                else:
-                    for i, (_seg, mask) in enumerate(component_info):
-                        res[i].append(val[mask_to_pos[mask]])
+                for i, (_seg, mask) in enumerate(component_info):
+                    res[i].append(val[mask_to_pos[mask]])
         return res
 
 
@@ -227,7 +256,7 @@ class MeasurementProfile:
 
     def need_mask(self, tree):
         if isinstance(tree, Leaf):
-            return tree.area == AreaType.Mask or tree.area == AreaType.Mask_without_ROI
+            return tree.area in [AreaType.Mask, AreaType.Mask_without_ROI]
         return self.need_mask(tree.left) or self.need_mask(tree.right)
 
     def _need_mask_without_segmentation(self, tree):
@@ -245,7 +274,7 @@ class MeasurementProfile:
 
         left_par, left_area = self._get_par_component_and_area_type(tree.left)
         right_par, right_area = self._get_par_component_and_area_type(tree.left)
-        if PerComponent.Yes == left_par or PerComponent.Yes == right_par:
+        if PerComponent.Yes in [left_par, right_par]:
             res_par = PerComponent.Yes
         else:
             res_par = PerComponent.No
@@ -265,34 +294,29 @@ class MeasurementProfile:
         return resp
 
     def __str__(self):
-        text = "Set name: {}\n".format(self.name)
+        text = f"Set name: {self.name}\n"
         if self.name_prefix != "":
-            text += "Name prefix: {}\n".format(self.name_prefix)
+            text += f"Name prefix: {self.name_prefix}\n"
         text += "Measurements list:\n"
         for el in self.chosen_fields:
-            text += "{}\n".format(el.name)
+            text += f"{el.name}\n"
         return text
 
     def get_component_info(self, unit: Units):
         """
         :return: list[((str, str), bool)]
         """
-        res = []
         # Fixme remove binding to 3 dimensions
-        for el in self.chosen_fields:
-            res.append(
-                (
-                    (self.name_prefix + el.name, el.get_unit(unit, 3)),
-                    self._is_component_measurement(el.calculation_tree),
-                )
+        return [
+            (
+                (self.name_prefix + el.name, el.get_unit(unit, 3)),
+                self._is_component_measurement(el.calculation_tree),
             )
-        return res
+            for el in self.chosen_fields
+        ]
 
     def is_any_mask_measurement(self):
-        for el in self.chosen_fields:
-            if self.need_mask(el.calculation_tree):
-                return True
-        return False
+        return any(self.need_mask(el.calculation_tree) for el in self.chosen_fields)
 
     def _is_component_measurement(self, node):
         if isinstance(node, Leaf):
@@ -321,12 +345,14 @@ class MeasurementProfile:
         kw["help_dict"] = help_dict
         kw["_area"] = node.area
         kw["_per_component"] = node.per_component
-        kw["_cache"] = True
+        kw["_cache"] = True  # TODO remove cache argument
         kw["area_array"] = kw[area_type_dict[area_type]]
+        kw["_component_num"] = NO_COMPONENT
         if node.per_component == PerComponent.No:
             val = method.calculate_property(**kw)
         else:
-            kw["_cache"] = False
+            # TODO use cache for per component calculate
+            # kw["_cache"] = False
             val = []
             area_array = kw["area_array"]
             if area_type == AreaType.ROI:
@@ -335,6 +361,7 @@ class MeasurementProfile:
                 components = segmentation_mask_map.mask_components
             for i in components:
                 kw["area_array"] = area_array == i
+                kw["_component_num"] = i
                 val.append(method.calculate_property(**kw))
             val = np.array(val)
             if node.per_component == PerComponent.Mean:
@@ -346,7 +373,7 @@ class MeasurementProfile:
     ) -> Tuple[Union[float, np.ndarray], symbols, AreaType]:
         method: MeasurementMethodBase = MEASUREMENT_DICT[node.name]
 
-        hash_str = hash_fun_call_name(method, node.dict, node.area, node.per_component, node.channel)
+        hash_str = hash_fun_call_name(method, node.dict, node.area, node.per_component, node.channel, NO_COMPONENT)
         area_type = method.area_type(node.area)
         if hash_str in help_dict:
             val = help_dict[hash_str]
@@ -364,7 +391,7 @@ class MeasurementProfile:
         left_res, left_unit, left_area = self.calculate_tree(node.left, segmentation_mask_map, help_dict, kwargs)
         right_res, right_unit, right_area = self.calculate_tree(node.right, segmentation_mask_map, help_dict, kwargs)
         if node.op != "/":
-            raise ValueError("Wrong measurement: {}".format(node))
+            raise ValueError(f"Wrong measurement: {node}")
         if isinstance(left_res, np.ndarray) and isinstance(right_res, np.ndarray) and left_area != right_area:
             area_set = {left_area, right_area}
             if area_set == {AreaType.ROI, AreaType.Mask_without_ROI}:  # pragma: no cover
@@ -439,13 +466,23 @@ class MeasurementProfile:
         return ComponentsInfo(components, mask_components, res)
 
     def get_component_and_area_info(self) -> List[Tuple[PerComponent, AreaType]]:
-        """For each measurement check if is per component and in which types """
+        """For each measurement check if is per component and in which types"""
         res = []
         for el in self.chosen_fields:
             tree = el.calculation_tree
             res.append(self._get_par_component_and_area_type(tree))
         return res
 
+    def get_segmentation_mask_map(self, image: Image, roi: Union[np.ndarray, ROIInfo], time: int = 0) -> ComponentsInfo:
+        def get_time(array: np.ndarray):
+            if array is not None and array.ndim == 4:
+                return array.take(time, axis=image.time_pos)
+            return array
+
+        return self.get_segmentation_to_mask_component(
+            get_time(roi if isinstance(roi, np.ndarray) else roi.roi), get_time(image.mask)
+        )
+
     def calculate(
         self,
         image: Image,
@@ -468,6 +505,45 @@ class MeasurementProfile:
         :return: measurements
         """
 
+        segmentation_mask_map = self.get_segmentation_mask_map(image, roi, time)
+        result = MeasurementResult(segmentation_mask_map)
+        range_changed(0, len(self.chosen_fields))
+        for i, (name, data) in enumerate(
+            self.calculate_yield(
+                image=image,
+                channel_num=channel_num,
+                roi=roi,
+                result_units=result_units,
+                segmentation_mask_map=segmentation_mask_map,
+                time=time,
+            ),
+            start=1,
+        ):
+            result[name] = data
+            step_changed(i)
+
+        return result
+
+    def calculate_yield(
+        self,
+        image: Image,
+        channel_num: int,
+        roi: Union[np.ndarray, ROIInfo],
+        result_units: Units,
+        segmentation_mask_map: ComponentsInfo,
+        time: int = 0,
+    ) -> Generator[MeasurementResultInputType, None, None]:
+        """
+        Calculate measurements on given set of parameters
+
+        :param image: image on which measurements should be calculated
+        :param roi: array with segmentation labeled as positive integers
+        :param result_units: units which should be used to present results.
+        :param segmentation_mask_map: information which component of roi belongs to which mask component.
+        :param time: which data point should be measured
+        :return: measurements
+        """
+
         def get_time(array: np.ndarray):
             if array is not None and array.ndim == 4:
                 return array.take(time, axis=image.time_pos)
@@ -475,7 +551,7 @@ class MeasurementProfile:
 
         if self._need_mask and image.mask is None:
             raise ValueError("measurement need mask")
-        channel = image.get_channel(channel_num).astype(np.float)
+        channel = image.get_channel(channel_num).astype(float)
         cache_dict = {}
         result_scalar = UNIT_SCALE[result_units.value]
         roi_alternative = {}
@@ -492,8 +568,6 @@ class MeasurementProfile:
             "roi_alternative": roi_alternative,
             "roi_annotation": roi.annotations if isinstance(roi, ROIInfo) else {},
         }
-        segmentation_mask_map = self.get_segmentation_to_mask_component(kw["segmentation"], kw["mask"])
-        result = MeasurementResult(segmentation_mask_map)
         for num in self.get_channels_num():
             kw["channel_{num}"] = get_time(image.get_channel(num))
         if any(self._need_mask_without_segmentation(el.calculation_tree) for el in self.chosen_fields):
@@ -501,14 +575,9 @@ class MeasurementProfile:
             mm[kw["segmentation"] > 0] = 0
             kw["mask_without_segmentation"] = mm
 
-        range_changed(0, len(self.chosen_fields))
-        for i, entry in enumerate(self.chosen_fields):
-            step_changed(i)
-            result[self.name_prefix + entry.name] = self._calc_single_field(
-                entry, segmentation_mask_map, cache_dict, kw, result_units
-            )
-
-        return result
+        for entry in self.chosen_fields:
+            name = self.name_prefix + entry.name
+            yield name, self._calc_single_field(entry, segmentation_mask_map, cache_dict, kw, result_units)
 
     def _calc_single_field(
         self,
@@ -527,8 +596,10 @@ class MeasurementProfile:
             return val, str(unit).format(str(result_units)), component_and_area
         except ZeroDivisionError:  # pragma: no cover
             return "Div by zero", "", component_and_area
-        except TypeError:  # pragma: no cover
-            return "None div", "", component_and_area
+        except TypeError as e:  # pragma: no cover
+            if e.args[0].startswith("unsupported operand type(s) for /:"):
+                return "None div", "", component_and_area
+            raise e
         except AttributeError:  # pragma: no cover
             return "No attribute", "", component_and_area
         except ProhibitedDivision as e:  # pragma: no cover
@@ -552,8 +623,7 @@ def calculate_main_axis(area_array: np.ndarray, channel: np.ndarray, voxel_size)
         positions[-i] *= v
         positions[-i] -= center_of_mass[i - 1]
     centered = np.dot(orientation_matrix.T, positions)
-    size = np.max(centered, axis=1) - np.min(centered, axis=1)
-    return size
+    return np.max(centered, axis=1) - np.min(centered, axis=1)
 
 
 def get_main_axis_length(
@@ -564,7 +634,9 @@ def get_main_axis_length(
         help_dict: Dict = kwargs["help_dict"]
         _area: AreaType = kwargs["_area"]
         _per_component: PerComponent = kwargs["_per_component"]
-        hash_name = hash_fun_call_name(calculate_main_axis, {}, _area, _per_component, kwargs["channel_num"])
+        hash_name = hash_fun_call_name(
+            calculate_main_axis, {}, _area, _per_component, kwargs["channel_num"], kwargs["_component_num"]
+        )
         if hash_name not in help_dict:
             help_dict[hash_name] = calculate_main_axis(area_array, channel, [x * result_scalar for x in voxel_size])
         return help_dict[hash_name][index]
@@ -578,6 +650,7 @@ def hash_fun_call_name(
     area: AreaType,
     per_component: PerComponent,
     channel: Channel,
+    components_num: int,
 ) -> str:
     """
     Calculate string for properly cache measurements result.
@@ -593,7 +666,7 @@ def hash_fun_call_name(
         fun_name = f"{fun.__module__}.{fun.__name__}"
     else:
         fun_name = fun.__name__
-    return "{}: {} # {} & {} * {}".format(fun_name, arguments, area, per_component, channel)
+    return f"{fun_name}: {arguments} # {area} & {per_component} * {channel} ^ {components_num}"
 
 
 class Volume(MeasurementMethodBase):
@@ -655,7 +728,7 @@ def iterative_double_normal(points_positions: np.ndarray):
     delta = 0
     dn = 0, 0
     point_index = 0
-    points_array = np.ones(points_positions.shape[0], dtype=np.bool)
+    points_array = np.ones(points_positions.shape[0], dtype=bool)
     while True:
         dn_r, delta_r = double_normal(point_index, points_positions, points_array)
         if delta_r > delta:
@@ -686,7 +759,7 @@ class Diameter(MeasurementMethodBase):
 
     @staticmethod
     def calculate_property(area_array, voxel_size, result_scalar, **_):  # pylint: disable=W0221
-        pos = np.transpose(np.nonzero(get_border(area_array))).astype(np.float)
+        pos = np.transpose(np.nonzero(get_border(area_array))).astype(float)
         if pos.size == 0:
             return 0
         for i, val in enumerate((x * result_scalar for x in reversed(voxel_size)), start=1):
@@ -945,14 +1018,18 @@ class Compactness(MeasurementMethodBase):
         cache = cache and "_per_component" in kwargs
         if cache:
             help_dict = kwargs["help_dict"]
-            border_hash_str = hash_fun_call_name(Surface, {}, kwargs["_area"], kwargs["_per_component"], Channel(-1))
+            border_hash_str = hash_fun_call_name(
+                Surface, {}, kwargs["_area"], kwargs["_per_component"], Channel(-1), kwargs["_component_num"]
+            )
             if border_hash_str not in help_dict:
                 border_surface = Surface.calculate_property(**kwargs)
                 help_dict[border_hash_str] = border_surface
             else:
                 border_surface = help_dict[border_hash_str]
 
-            volume_hash_str = hash_fun_call_name(Volume, {}, kwargs["_area"], kwargs["_per_component"], Channel(-1))
+            volume_hash_str = hash_fun_call_name(
+                Volume, {}, kwargs["_area"], kwargs["_per_component"], Channel(-1), kwargs["_component_num"]
+            )
 
             if volume_hash_str not in help_dict:
                 volume = Volume.calculate_property(**kwargs)
@@ -980,15 +1057,19 @@ class Sphericity(MeasurementMethodBase):
             help_dict = kwargs["help_dict"]
         else:
             help_dict = {}
-            kwargs.update({"_area": AreaType.ROI, "_per_component": PerComponent.No})
-        volume_hash_str = hash_fun_call_name(Volume, {}, kwargs["_area"], kwargs["_per_component"], Channel(-1))
+            kwargs.update({"_area": AreaType.ROI, "_per_component": PerComponent.No, "_component_num": NO_COMPONENT})
+        volume_hash_str = hash_fun_call_name(
+            Volume, {}, kwargs["_area"], kwargs["_per_component"], Channel(-1), kwargs["_component_num"]
+        )
         if volume_hash_str not in help_dict:
             volume = Volume.calculate_property(**kwargs)
             help_dict[volume_hash_str] = volume
         else:
             volume = help_dict[volume_hash_str]
 
-        diameter_hash_str = hash_fun_call_name(Diameter, {}, kwargs["_area"], kwargs["_per_component"], Channel(-1))
+        diameter_hash_str = hash_fun_call_name(
+            Diameter, {}, kwargs["_area"], kwargs["_per_component"], Channel(-1), kwargs["_component_num"]
+        )
         if diameter_hash_str not in help_dict:
             diameter_val = Diameter.calculate_property(**kwargs)
             help_dict[diameter_hash_str] = diameter_val
@@ -1115,7 +1196,7 @@ class DistanceMaskSegmentation(MeasurementMethodBase):
     @staticmethod
     def calculate_points(channel, area_array, voxel_size, result_scalar, point_type: DistancePoint) -> np.ndarray:
         if point_type == DistancePoint.Border:
-            area_pos = np.transpose(np.nonzero(get_border(area_array))).astype(np.float)
+            area_pos = np.transpose(np.nonzero(get_border(area_array))).astype(float)
             area_pos += 0.5
             for i, val in enumerate((x * result_scalar for x in reversed(voxel_size)), start=1):
                 area_pos[:, -i] *= val
@@ -1148,7 +1229,7 @@ class DistanceMaskSegmentation(MeasurementMethodBase):
             return 0
         mask_pos = cls.calculate_points(channel, mask, voxel_size, result_scalar, distance_from_mask)
         seg_pos = cls.calculate_points(channel, area_array, voxel_size, result_scalar, distance_to_segmentation)
-        if mask_pos.shape[0] == 1 or seg_pos.shape[0] == 1:
+        if 1 in {mask_pos.shape[0], seg_pos.shape[0]}:
             return np.min(cdist(mask_pos, seg_pos))
 
         min_val = np.inf
@@ -1241,34 +1322,95 @@ class SplitOnPartPixelBrightnessSum(MeasurementMethodBase):
         return True
 
 
+HARALIC_FEATURES = """AngularSecondMoment Contrast Correlation Variance
+InverseDifferenceMoment SumAverage SumVariance SumEntropy Entropy
+DifferenceVariance DifferenceEntropy InfoMeas1 InfoMeas2""".split()
+
+
+def _rescale_image(data: np.ndarray):
+    if data.dtype == np.uint8:
+        return data
+    min_val = data.min()
+    max_val = data.max()
+    return ((data - min_val) / ((max_val - min_val) / 255)).astype(np.uint8)
+
+
+class Haralick(MeasurementMethodBase):
+    @classmethod
+    def get_units(cls, ndim) -> symbols:
+        return "1"
+
+    text_info = "Haralick", "Calculate Haralick features"
+
+    @classmethod
+    def get_fields(cls):
+        return [
+            AlgorithmProperty("feature", "Feature", HARALIC_FEATURES[0], possible_values=HARALIC_FEATURES),
+            AlgorithmProperty("distance", "Distance", 1, options_range=(1, 10)),
+        ]
+
+    @classmethod
+    def need_channel(cls):
+        return True
+
+    @classmethod
+    def calculate_property(
+        cls, area_array, channel, distance, feature, _cache=False, **kwargs
+    ):  # pylint: disable=W0221
+        _cache = _cache and "_area" in kwargs and "_per_component" in kwargs
+        if _cache:
+            help_dict: Dict = kwargs["help_dict"]
+            _area: AreaType = kwargs["_area"]
+            _per_component: PerComponent = kwargs["_per_component"]
+            hash_name = hash_fun_call_name(
+                Haralick, {"distance": distance}, _area, _per_component, kwargs["channel_num"], kwargs["_component_num"]
+            )
+            if hash_name not in help_dict:
+                help_dict[hash_name] = cls.calculate_haralick(channel, area_array, distance)
+            return help_dict[hash_name][HARALIC_FEATURES.index(feature)]
+
+        res = cls.calculate_haralick(channel, area_array, distance)
+        return res[HARALIC_FEATURES.index(feature)]
+
+    @staticmethod
+    def calculate_haralick(channel, area_array, distance):
+        data = channel.copy()
+        data[area_array == 0] = 0
+        data = _rescale_image(data.squeeze())
+        return haralick(data, distance=distance, ignore_zeros=True, return_mean=True)
+
+
 def pixel_volume(spacing, result_scalar):
     return reduce((lambda x, y: x * y), [x * result_scalar for x in spacing])
 
 
 def calculate_volume_surface(volume_mask, voxel_size):
-    border_surface = 0
     surf_im: np.ndarray = np.array(volume_mask).astype(np.uint8).squeeze()
-    for ax in range(surf_im.ndim):
-        border_surface += (
+    return sum(
+        (
             np.count_nonzero(
                 np.logical_xor(
                     surf_im.take(np.arange(surf_im.shape[ax] - 1), axis=ax),
                     surf_im.take(np.arange(surf_im.shape[ax] - 1) + 1, axis=ax),
                 )
             )
-            * reduce(lambda x, y: x * y, [voxel_size[x] for x in range(surf_im.ndim) if x != ax])
+            * reduce(
+                lambda x, y: x * y,
+                [voxel_size[x] for x in range(surf_im.ndim) if x != ax],
+            )
         )
-    return border_surface
+        for ax in range(surf_im.ndim)
+    )
 
 
 def get_border(array):
-    if array.dtype == np.bool:
+    if array.dtype == bool:
         array = array.astype(np.uint8)
     return SimpleITK.GetArrayFromImage(SimpleITK.LabelContour(SimpleITK.GetImageFromArray(array)))
 
 
 def calc_diam(array, voxel_size):  # pragma: no cover
-    pos = np.transpose(np.nonzero(array)).astype(np.float)
+    pos = np.transpose(np.nonzero(array)).astype(float)
     for i, val in enumerate(voxel_size):
         pos[:, i] *= val
     diam = 0
@@ -1301,6 +1443,7 @@ MEASUREMENT_DICT = Register(
     SplitOnPartVolume,
     SplitOnPartPixelBrightnessSum,
     Voxels,
+    Haralick,
     suggested_base_class=MeasurementMethodBase,
 )
 """Register with all measurements algorithms"""
