@@ -73,7 +73,7 @@ def convex_fill(array: np.ndarray):
             continue
         lower_bound = np.min(points, axis=1)
         upper_bound = np.max(points, axis=1)
-        cut_area = tuple([slice(x, y + 1) for x, y in zip(lower_bound, upper_bound)])
+        cut_area = tuple(slice(x, y + 1) for x, y in zip(lower_bound, upper_bound))
         if array.ndim == 3:
             cut_area = (slice(None),) + cut_area[1:]
         component = component[cut_area]
