@@ -1,6 +1,9 @@
 import copy
 import typing
 
+import numpy as np
+from napari.utils import Colormap
+
 from PartSegCore.algorithm_describe_base import ROIExtractionProfile
 
 from .class_generator import SerializeClassEncoder, serialize_hook
@@ -77,7 +80,7 @@ class ProfileDict:
                 curr_dict = curr_dict[key]
             except KeyError:
                 for key2 in key_path[i:-1]:
-                    curr_dict[key2] = dict()
+                    curr_dict[key2] = {}
                     curr_dict = curr_dict[key2]
                     break
         curr_dict[key_path[-1]] = value
@@ -146,6 +149,18 @@ class ProfileEncoder(SerializeClassEncoder):
             return {"__RadiusType__": True, "value": o.value}
         if isinstance(o, ROIExtractionProfile):
             return {"__SegmentationProfile__": True, "name": o.name, "algorithm": o.algorithm, "values": o.values}
+        if isinstance(o, Colormap):
+            return {
+                "__Colormap__": True,
+                "name": o.name,
+                "colors": o.colors.tolist(),
+                "interpolation": o.interpolation,
+                "controls": o.controls.tolist(),
+            }
+        if isinstance(o, np.integer):
+            return int(o)
+        if isinstance(o, np.floating):
+            return float(o)
         return super().default(o)
 
 
@@ -173,12 +188,31 @@ def profile_hook(dkt):
         del dkt["__SegmentationProfile__"]
         res = ROIExtractionProfile(**dkt)
         return res
-    if "__Serializable__" in dkt and dkt["__subtype__"] == "HistoryElement" and "algorithm_name" in dkt:
+    if (
+        "__Serializable__" in dkt and dkt["__subtype__"] == "HistoryElement" and "algorithm_name" in dkt
+    ):  # pragma: no cover
+        # old code fix
         name = dkt["algorithm_name"]
         par = dkt["algorithm_values"]
         del dkt["algorithm_name"]
         del dkt["algorithm_values"]
         dkt["segmentation_parameters"] = {"algorithm_name": name, "values": par}
+    if "__Serializable__" in dkt and dkt["__subtype__"] == "PartSegCore.color_image.base_colors.ColorMap":
+        positions, colors = list(zip(*dkt["colormap"]))
+        return Colormap(colors, controls=positions)
+    if "__Serializable__" in dkt and dkt["__subtype__"] == "PartSegCore.color_image.base_colors.ColorPosition":
+        return (dkt["color_position"], dkt["color"])
+    if "__Serializable__" in dkt and dkt["__subtype__"] == "PartSegCore.color_image.base_colors.Color":
+        return (dkt["red"] / 255, dkt["green"] / 255, dkt["blue"] / 255)
+    if "__Colormap__" in dkt:
+        del dkt["__Colormap__"]
+        if dkt["controls"][0] != 0:
+            dkt["controls"].insert(0, 0)
+            dkt["colors"].insert(0, dkt["colors"][0])
+        if dkt["controls"][-1] != 1:
+            dkt["controls"].append(1)
+            dkt["colors"].append(dkt["colors"][-1])
+        return Colormap(**dkt)
 
     return serialize_hook(dkt)
 
@@ -193,7 +227,4 @@ def check_loaded_dict(dkt) -> bool:
         return True
     if "__error__" in dkt:
         return False
-    for val in dkt.values():
-        if not check_loaded_dict(val):
-            return False
-    return True
+    return all(check_loaded_dict(val) for val in dkt.values())
