@@ -10,14 +10,15 @@ from pathlib import Path
 
 import numpy as np
 import tifffile
+from napari.plugins._builtins import napari_write_points
 
 from PartSegImage import GenericImageReader, Image, ImageWriter, TiffImageReader
-from PartSegImage.image import reduce_array
+from PartSegImage.image import FRAME_THICKNESS, reduce_array
 
 from ..algorithm_describe_base import AlgorithmProperty, Register, ROIExtractionProfile
 from ..io_utils import (
-    HistoryElement,
     LoadBase,
+    LoadPoints,
     SaveBase,
     SaveMaskAsTiff,
     SaveROIAsNumpy,
@@ -32,7 +33,7 @@ from ..io_utils import (
     tar_to_buff,
 )
 from ..json_hooks import ProfileEncoder
-from ..project_info import ProjectInfoBase
+from ..project_info import AdditionalLayerDescription, HistoryElement, ProjectInfoBase
 from ..roi_info import ROIInfo
 
 
@@ -45,8 +46,8 @@ class MaskProjectTuple(ProjectInfoBase):
     :ivar typing.Union[Image,str,None] ~.image: image which is proceeded in given segmentation.
         If :py:class:`str` then it is path to image on drive
     :ivar typing.Optional[np.ndarray] ~.mask: Mask limiting segmentation area.
-    :ivar typing.Optional[np.ndarray] ~.segmentation: Segmentation array.
-    :ivar SegmentationInfo ~.segmentation_info: segmentation description
+    :ivar typing.Optional[np.ndarray] ~.roi: ROI array.
+    :ivar SegmentationInfo ~.roi_info: ROI description
     :ivar typing.List[int] ~.selected_components: list of selected components
     :ivar typing.Dict[int,typing.Optional[SegmentationProfile]] ~.segmentation_parameters:
         For each component description set of parameters used for segmentation
@@ -59,8 +60,8 @@ class MaskProjectTuple(ProjectInfoBase):
     file_path: str
     image: typing.Union[Image, str, None]
     mask: typing.Optional[np.ndarray] = None
-    roi: typing.Optional[np.ndarray] = None
     roi_info: ROIInfo = ROIInfo(None)
+    additional_layers: typing.Dict[str, AdditionalLayerDescription] = dataclasses.field(default_factory=dict)
     selected_components: typing.List[int] = dataclasses.field(default_factory=list)
     roi_extraction_parameters: typing.Dict[int, typing.Optional[ROIExtractionProfile]] = dataclasses.field(
         default_factory=dict
@@ -68,6 +69,7 @@ class MaskProjectTuple(ProjectInfoBase):
     history: typing.List[HistoryElement] = dataclasses.field(default_factory=list)
     errors: str = ""
     spacing: typing.Optional[typing.List[float]] = None
+    points: typing.Optional[np.ndarray] = None
 
     def get_raw_copy(self):
         return MaskProjectTuple(self.file_path, self.image.substitute(mask=None))
@@ -103,19 +105,22 @@ def save_stack_segmentation(
             spacing = segmentation_info.image.spacing
         else:
             spacing = parameters.get("spacing", (10 ** -6, 10 ** -6, 10 ** -6))
-        segmentation_image = Image(segmentation_info.roi, spacing, axes_order=Image.axis_order.replace("C", ""))
+        segmentation_image = Image(
+            segmentation_info.roi_info.roi, spacing, axes_order=Image.axis_order.replace("C", "")
+        )
         try:
             ImageWriter.save(segmentation_image, segmentation_buff)
         except ValueError:
             segmentation_buff.seek(0)
-            tifffile.imwrite(segmentation_buff, segmentation_info.roi, compress=9)
+            tifffile.imwrite(segmentation_buff, segmentation_info.roi_info.roi)
         segmentation_tar = get_tarinfo("segmentation.tif", segmentation_buff)
         tar_file.addfile(segmentation_tar, fileobj=segmentation_buff)
         step_changed(3)
         metadata = {
             "components": [int(x) for x in segmentation_info.selected_components],
             "parameters": {str(k): v for k, v in segmentation_info.roi_extraction_parameters.items()},
-            "shape": segmentation_info.roi.shape,
+            "shape": segmentation_info.roi_info.roi.shape,
+            "annotations": segmentation_info.roi_info.annotations,
         }
         if isinstance(segmentation_info.image, Image):
             file_path = segmentation_info.image.file_path
@@ -134,12 +139,17 @@ def save_stack_segmentation(
         step_changed(4)
         if segmentation_info.mask is not None:
             mask = segmentation_info.mask
-            if mask.dtype == np.bool:
+            if mask.dtype == bool:
                 mask = mask.astype(np.uint8)
             mask_buff = BytesIO()
-            tifffile.imwrite(mask_buff, mask, compress=9)
+            tifffile.imwrite(mask_buff, mask)
             mask_tar = get_tarinfo("mask.tif", mask_buff)
             tar_file.addfile(mask_tar, fileobj=mask_buff)
+        if segmentation_info.roi_info.alternative:
+            alternative_buff = BytesIO()
+            np.savez(alternative_buff, **segmentation_info.roi_info.alternative)
+            alternative_tar = get_tarinfo("alternative.npz", alternative_buff)
+            tar_file.addfile(alternative_tar, fileobj=alternative_buff)
         step_changed(5)
         el_info = []
         for i, hist in enumerate(segmentation_info.history):
@@ -147,14 +157,15 @@ def save_stack_segmentation(
                 {
                     "index": i,
                     "mask_property": hist.mask_property,
-                    "segmentation_parameters": hist.segmentation_parameters,
+                    "segmentation_parameters": hist.roi_extraction_parameters,
+                    "annotations": hist.annotations,
                 }
             )
             hist.arrays.seek(0)
             hist_info = get_tarinfo(f"history/arrays_{i}.npz", hist.arrays)
             hist.arrays.seek(0)
             tar_file.addfile(hist_info, hist.arrays)
-        if len(el_info) > 0:
+        if el_info:
             hist_str = json.dumps(el_info, cls=ProfileEncoder)
             hist_buff = BytesIO(hist_str.encode("utf-8"))
             tar_algorithm = get_tarinfo("history/history.json", hist_buff)
@@ -192,20 +203,24 @@ def load_stack_segmentation(file_data: typing.Union[str, Path], range_changed=No
         segmentation_buff.write(segmentation_tar.read())
         step_changed(3)
         segmentation_buff.seek(0)
-        segmentation = segmentation_load_fun(segmentation_buff)
-        if isinstance(segmentation, Image):
-            spacing = segmentation.spacing
-            segmentation = segmentation.get_channel(0)
+        roi = segmentation_load_fun(segmentation_buff)
+        if isinstance(roi, Image):
+            spacing = roi.spacing
+            roi = roi.get_channel(0)
         else:
             spacing = None
-        segmentation = reduce_array(segmentation)
         step_changed(4)
         if "mask.tif" in tar_file.getnames():
             mask = tifffile.imread(tar_to_buff(tar_file, "mask.tif"))
             if np.max(mask) == 1:
-                mask = mask.astype(np.bool)
+                mask = mask.astype(bool)
         else:
             mask = None
+        if "alternative.npz" in tar_file.getnames():
+            alternative = np.load(tar_to_buff(tar_file, "alternative.npz"))
+        else:
+            alternative = {}
+        roi_info = ROIInfo(reduce_array(roi), annotations=metadata.get("annotations", {}), alternative=alternative)
         step_changed(5)
         history = []
         try:
@@ -217,9 +232,10 @@ def load_stack_segmentation(file_data: typing.Union[str, Path], range_changed=No
                 history_buffer.seek(0)
                 history.append(
                     HistoryElement(
-                        segmentation_parameters=el["segmentation_parameters"],
+                        roi_extraction_parameters=el["segmentation_parameters"],
                         mask_property=el["mask_property"],
                         arrays=history_buffer,
+                        annotations=el.get("annotations", {}),
                     )
                 )
 
@@ -232,7 +248,7 @@ def load_stack_segmentation(file_data: typing.Union[str, Path], range_changed=No
     return MaskProjectTuple(
         file_path=file_data if isinstance(file_data, str) else "",
         image=metadata["base_file"] if "base_file" in metadata else None,
-        roi=segmentation,
+        roi_info=roi_info,
         selected_components=metadata["components"],
         mask=mask,
         roi_extraction_parameters=metadata["parameters"] if "parameters" in metadata else None,
@@ -242,10 +258,12 @@ def load_stack_segmentation(file_data: typing.Union[str, Path], range_changed=No
 
 
 def empty_fun(_a0=None, _a1=None):
-    pass
+    """
+    This is empty fun to pass as callback to for report.
+    """
 
 
-class LoadSegmentation(LoadBase):
+class LoadROI(LoadBase):
     """
     Load ROI segmentation data.
     """
@@ -262,16 +280,15 @@ class LoadSegmentation(LoadBase):
     def fix_parameters(profile: ROIExtractionProfile):
         if profile is None:
             return
-        if profile.algorithm == "Threshold" or profile.algorithm == "Auto Threshold":
-            if isinstance(profile.values["smooth_border"], bool):
-                if profile.values["smooth_border"] and "smooth_border_radius" in profile.values:
-                    profile.values["smooth_border"] = {
-                        "name": "Opening",
-                        "values": {"smooth_border_radius": profile.values["smooth_border_radius"]},
-                    }
-                    del profile.values["smooth_border_radius"]
-                else:
-                    profile.values["smooth_border"] = {"name": "None", "values": {}}
+        if (profile.algorithm in {"Threshold", "Auto Threshold"}) and isinstance(profile.values["smooth_border"], bool):
+            if profile.values["smooth_border"] and "smooth_border_radius" in profile.values:
+                profile.values["smooth_border"] = {
+                    "name": "Opening",
+                    "values": {"smooth_border_radius": profile.values["smooth_border_radius"]},
+                }
+                del profile.values["smooth_border_radius"]
+            else:
+                profile.values["smooth_border"] = {"name": "None", "values": {}}
         return profile
 
     @classmethod
@@ -331,10 +348,7 @@ class LoadROIParameters(LoadBase):
                 else:
                     parameters = defaultdict(
                         lambda: None,
-                        [
-                            (int(k), LoadSegmentation.fix_parameters(v))
-                            for k, v in project_metadata["parameters"].items()
-                        ],
+                        [(int(k), LoadROI.fix_parameters(v)) for k, v in project_metadata["parameters"].items()],
                     )
                 return MaskProjectTuple(file_path=file_data, image=None, roi_extraction_parameters=parameters)
 
@@ -343,7 +357,7 @@ class LoadROIParameters(LoadBase):
             project_metadata = load_metadata(tar_file.extractfile("metadata.json").read().decode("utf8"))
             parameters = defaultdict(
                 lambda: None,
-                [(int(k), LoadSegmentation.fix_parameters(v)) for k, v in project_metadata["parameters"].items()],
+                [(int(k), LoadROI.fix_parameters(v)) for k, v in project_metadata["parameters"].items()],
             )
         finally:
             if isinstance(file_data, (str, Path)):
@@ -372,21 +386,18 @@ class LoadROIImage(LoadBase):
         step_changed: typing.Callable[[int], typing.Any] = None,
         metadata: typing.Optional[dict] = None,
     ) -> MaskProjectTuple:
-        seg = LoadSegmentation.load(load_locations)
-        if len(load_locations) > 1:
-            base_file = load_locations[1]
-        else:
-            base_file = seg.image
+        seg = LoadROI.load(load_locations)
+        base_file = load_locations[1] if len(load_locations) > 1 else seg.image
         if base_file is None:
-            raise IOError("base file for segmentation not defined")
+            raise OSError("base file for segmentation not defined")
         if os.path.isabs(base_file):
             file_path = base_file
+        elif not isinstance(load_locations[0], str):
+            raise OSError(f"Cannot use relative path {base_file} for non path argument")
         else:
-            if not isinstance(load_locations[0], str):
-                raise IOError(f"Cannot use relative path {base_file} for non path argument")
             file_path = os.path.join(os.path.dirname(load_locations[0]), base_file)
         if not os.path.exists(file_path):
-            raise IOError(f"Base file for segmentation do not exists: {base_file} -> {file_path}")
+            raise OSError(f"Base file for segmentation do not exists: {base_file} -> {file_path}")
         if metadata is None:
             metadata = {"default_spacing": (10 ** -6, 10 ** -6, 10 ** -6)}
         image = GenericImageReader.read_image(
@@ -396,7 +407,9 @@ class LoadROIImage(LoadBase):
         )
         # noinspection PyProtectedMember
         # image.file_path = load_locations[0]
-        return dataclasses.replace(seg, file_path=image.file_path, image=image, roi=image.fit_array_to_image(seg.roi))
+        return dataclasses.replace(
+            seg, file_path=image.file_path, image=image, roi_info=seg.roi_info.fit_to_image(image)
+        )
 
 
 class LoadStackImage(LoadBase):
@@ -406,7 +419,7 @@ class LoadStackImage(LoadBase):
 
     @classmethod
     def get_name(cls):
-        return "Image(*.tif *.tiff *.lsm *.czi *.oib *.oif)"
+        return "Image(*.tif *.tiff *.lsm *.czi *.oib *.oif *.obsep)"
 
     @classmethod
     def get_short_name(cls):
@@ -505,9 +518,9 @@ class SaveROI(SaveBase):
 def save_components(
     image: Image,
     components: list,
-    segmentation: np.ndarray,
     dir_path: str,
-    segmentation_info: typing.Optional[ROIInfo] = None,
+    roi_info: ROIInfo,
+    points: typing.Optional[np.ndarray] = None,
     range_changed=None,
     step_changed=None,
 ):
@@ -516,16 +529,31 @@ def save_components(
     if step_changed is None:
         step_changed = empty_fun
 
-    segmentation = image.fit_array_to_image(segmentation)
-
-    if segmentation_info is None:
-        segmentation_info = ROIInfo(segmentation)
+    roi_info = roi_info.fit_to_image(image)
     os.makedirs(dir_path, exist_ok=True)
 
     file_name = os.path.splitext(os.path.basename(image.file_path))[0]
+    points_casted = None
+    important_axis = "XY" if image.is_2d else "XYZ"
+    index_to_frame_points = image.calc_index_to_frame(image.axis_order, important_axis)
+    if points is not None:
+        points_casted = points.astype(np.uint16)
+
     range_changed(0, 2 * len(components))
     for i in components:
-        im = image.cut_image(segmentation == i, replace_mask=True)
+        components_mark = np.array(roi_info.roi == i)
+        im = image.cut_image(components_mark, replace_mask=True)
+        if points is not None and points_casted is not None:
+            points_mask = components_mark[tuple(points_casted.T)]
+            filtered_points = points[points_mask]
+            filtered_points[:, 1] = np.round(filtered_points[:, 1])
+            lower_bound = np.min(np.nonzero(components_mark), axis=1)
+            for j in index_to_frame_points:
+                lower_bound[j] -= FRAME_THICKNESS
+            napari_write_points(
+                os.path.join(dir_path, f"{file_name}_component{i}.csv"), filtered_points - lower_bound, {}
+            )
+
         # print(f"[run] {im}")
         ImageWriter.save(im, os.path.join(dir_path, f"{file_name}_component{i}.tif"))
         step_changed(2 * i + 1)
@@ -554,9 +582,9 @@ class SaveComponents(SaveBase):
         save_components(
             project_info.image,
             project_info.selected_components,
-            project_info.roi,
             save_location,
             project_info.roi_info,
+            project_info.points,
             range_changed,
             step_changed,
         )
@@ -614,12 +642,12 @@ class LoadROIFromTIFF(LoadBase):
         metadata: typing.Optional[dict] = None,
     ) -> typing.Union[ProjectInfoBase, typing.List[ProjectInfoBase]]:
         image = TiffImageReader.read_image(load_locations[0])
-        segmentation = image.get_channel(0)
+        roi = image.get_channel(0)
         return MaskProjectTuple(
             file_path=load_locations[0],
             image=None,
-            segmentation=segmentation,
-            segmentation_parameters=defaultdict(lambda: None),
+            roi_info=ROIInfo(roi),
+            roi_extraction_parameters=defaultdict(lambda: None),
         )
 
     @classmethod
@@ -644,7 +672,7 @@ class UpdateLoadedMetadataMask(UpdateLoadedMetadataBase):
     @classmethod
     def update_segmentation_profile(cls, profile_data: ROIExtractionProfile) -> ROIExtractionProfile:
         profile_data = super().update_segmentation_profile(profile_data)
-        if profile_data.algorithm == "Threshold" or profile_data.algorithm == "Auto Threshold":
+        if profile_data.algorithm in {"Threshold", "Auto Threshold"}:
             if isinstance(profile_data.values["smooth_border"], bool):
                 if profile_data.values["smooth_border"]:
                     profile_data.values["smooth_border"] = {
@@ -661,7 +689,9 @@ class UpdateLoadedMetadataMask(UpdateLoadedMetadataBase):
         return profile_data
 
 
-load_dict = Register(LoadStackImage, LoadROIImage, LoadStackImageWithMask, class_methods=LoadBase.need_functions)
+load_dict = Register(
+    LoadStackImage, LoadROIImage, LoadStackImageWithMask, LoadPoints, class_methods=LoadBase.need_functions
+)
 save_parameters_dict = Register(SaveParametersJSON, class_methods=SaveBase.need_functions)
 save_components_dict = Register(SaveComponents, class_methods=SaveBase.need_functions)
 save_segmentation_dict = Register(
