@@ -7,7 +7,6 @@ from PartSegImage.image import minimal_dtype
 
 from .class_generator import BaseSerializableClass
 from .image_operations import RadiusType, dilate, erode
-from .project_info import ProjectInfoBase
 
 
 class MaskProperty(BaseSerializableClass):
@@ -50,6 +49,17 @@ class MaskProperty(BaseSerializableClass):
             + f"reversed mask={self.reversed_mask})"
         )
 
+    def __eq__(self, other):
+        return (
+            isinstance(other, MaskProperty)
+            and self.dilate == other.dilate
+            and self.dilate_radius == other.dilate_radius
+            and self.fill_holes == other.fill_holes
+            and self.max_holes_size == other.max_holes_size
+            and self.clip_to_mask == other.clip_mask
+            and self.reversed_mask == other.reverse_mask
+        )
+
     @classmethod
     def simple_mask(cls) -> "MaskProperty":
         """
@@ -77,30 +87,9 @@ def mp_eq(self: MaskProperty, other: MaskProperty):
 MaskProperty.__eq__ = mp_eq
 
 
-def calculate_mask_from_project(
-    mask_description: MaskProperty, project: ProjectInfoBase, components: typing.Optional[typing.List[int]] = None
-) -> np.ndarray:
-    """
-    Function for calculate mask base on MaskProperty.
-    This function calls :py:func:`calculate_mask` with arguments from project.
-
-    :param MaskProperty mask_description: information how calculate mask
-    :param ProjectInfoBase project: project with information about segmentation
-    :param typing.Optional[typing.List[int]] components: If present inform which components
-        should be used when calculation mask, otherwise use all.
-    :return: new mask
-    :rtype: np.ndarray
-    """
-    try:
-        time_axis = project.image.time_pos
-    except AttributeError:
-        time_axis = None
-    return calculate_mask(mask_description, project.roi, project.mask, project.image.spacing, components, time_axis)
-
-
 def calculate_mask(
     mask_description: MaskProperty,
-    segmentation: np.ndarray,
+    roi: np.ndarray,
     old_mask: typing.Optional[np.ndarray],
     spacing: typing.Iterable[typing.Union[float, int]],
     components: typing.Optional[typing.List[int]] = None,
@@ -112,7 +101,7 @@ def calculate_mask(
     otherwise it is done after dilate
 
     :param MaskProperty mask_description: information how calculate mask
-    :param np.ndarray segmentation: array on which mask is calculated
+    :param np.ndarray roi: array on which mask is calculated
     :param typing.Optional[np.ndarray] old_mask: if in mask_description there is set to crop and old_mask is not None
         then final mask is clipped to this area
     :param typing.Iterable[typing.Union[float,int]] spacing: spacing of image. Needed for calculating radius of dilate
@@ -128,16 +117,13 @@ def calculate_mask(
     if mask_description.dilate == RadiusType.R2D:
         dilate_radius = dilate_radius[-2:]
     if components is not None:
-        components_num = max(np.max(segmentation), *components) + 1
+        components_num = max(np.max(roi), *components) + 1
         map_array = np.zeros(components_num, dtype=minimal_dtype(components_num))
         for el in components:
             map_array[el] = el
-        segmentation = map_array[segmentation]
+        roi = map_array[roi]
 
-    if mask_description.save_components:
-        mask = np.copy(segmentation)
-    else:
-        mask = np.array(segmentation > 0)
+    mask = np.copy(roi) if mask_description.save_components else np.array(roi > 0)
     if time_axis is None:
         return _calculate_mask(mask_description, dilate_radius, mask, old_mask)
     slices: typing.List[typing.Union[slice, int]] = [slice(None) for _ in range(mask.ndim)]
@@ -147,10 +133,9 @@ def calculate_mask(
     res = []
     for i in range(mask.shape[time_axis]):
         slices[time_axis] = i
-        _old_mask = old_mask[tuple(slices)] if old_mask is not None else None
-        res.append(
-            _calculate_mask(mask_description, dilate_radius, mask[tuple(slices)], _old_mask).reshape(final_shape)
-        )
+        t_slices = tuple(slices)
+        _old_mask = old_mask[t_slices] if old_mask is not None else None
+        res.append(_calculate_mask(mask_description, dilate_radius, mask[t_slices], _old_mask).reshape(final_shape))
     return np.concatenate(res, axis=time_axis)
 
 
@@ -185,11 +170,11 @@ def _cut_components(
             points = np.nonzero(mask == i)
             lower_bound = np.min(points, axis=1)
             upper_bound = np.max(points, axis=1)
-            new_cut = tuple([slice(x, y + 1) for x, y in zip(lower_bound, upper_bound)])
+            new_cut = tuple(slice(x, y + 1) for x, y in zip(lower_bound, upper_bound))
             new_size = [y - x + 1 + 2 * borders for x, y in zip(lower_bound, upper_bound)]
             if borders > 0:
                 res = np.zeros(new_size, dtype=image.dtype)
-                res_cut = tuple([slice(borders, x - borders) for x in res.shape])
+                res_cut = tuple(slice(borders, x - borders) for x in tuple(res.shape))
                 tmp_res = np.copy(image[new_cut])
                 tmp_res[mask[new_cut] != i] = 0
                 res[res_cut] = tmp_res
@@ -204,7 +189,7 @@ def _fill_holes(mask_description: MaskProperty, mask: np.ndarray) -> np.ndarray:
         return mask
     if mask_description.save_components:
         border = 1
-        res_slice = tuple([slice(border, -border) for _ in range(mask.ndim)])
+        res_slice = tuple(slice(border, -border) for _ in range(mask.ndim))
         mask_description_copy = mask_description.replace_(save_components=False)
         mask_prohibited = mask > 0
         for component, slice_arr, cmp_num in _cut_components(mask, mask, border):
@@ -218,7 +203,7 @@ def _fill_holes(mask_description: MaskProperty, mask: np.ndarray) -> np.ndarray:
             mask = fill_2d_holes_in_mask(mask, mask_description.max_holes_size)
         elif mask_description.fill_holes == RadiusType.R3D:
             mask = fill_holes_in_mask(mask, mask_description.max_holes_size)
-        mask = mask.astype(np.bool)
+        mask = mask.astype(bool)
     return mask
 
 
