@@ -15,14 +15,14 @@ def project_to_layers(project_info: typing.Union[ProjectTuple, MaskProjectTuple]
             res_layers.append(
                 (
                     project_info.image.get_channel(i),
-                    {"scale": scale, "name": f"channel {i}", "blending": "additive"},
+                    {"scale": scale, "name": project_info.image.channel_names[i], "blending": "additive"},
                     "image",
                 )
             )
-        if project_info.roi is not None:
+        if project_info.roi_info.roi is not None:
             res_layers.append(
                 (
-                    project_info.image.fit_array_to_image(project_info.roi),
+                    project_info.image.fit_array_to_image(project_info.roi_info.roi),
                     {"scale": scale, "name": "ROI"},
                     "labels",
                 )
@@ -40,10 +40,10 @@ def project_to_layers(project_info: typing.Union[ProjectTuple, MaskProjectTuple]
             scale = np.multiply(project_info.spacing, 10 ** 9)
         else:
             scale = None
-        if project_info.roi is not None:
+        if project_info.roi_info.roi is not None:
             res_layers.append(
                 (
-                    project_info.roi,
+                    project_info.roi_info.roi,
                     {"scale": scale, "name": "ROI"},
                     "labels",
                 )
