@@ -10,7 +10,8 @@ from PartSegImage import Image
 
 from ..algorithm_describe_base import AlgorithmDescribeBase, AlgorithmProperty, ROIExtractionProfile
 from ..image_operations import RadiusType
-from ..project_info import ProjectInfoBase
+from ..project_info import AdditionalLayerDescription, ProjectInfoBase
+from ..roi_info import ROIInfo
 from ..utils import numpy_repr
 
 
@@ -24,27 +25,6 @@ def calculate_operation_radius(radius, spacing, gauss_type):
     return radius
 
 
-@dataclass
-class AdditionalLayerDescription:
-    """
-    Dataclass
-
-    :ivar numpy.ndarray data: layer data
-    :ivar str layer_type: napari layer type
-    :ivar str name: layer name
-    """
-
-    data: np.ndarray
-    layer_type: str
-    name: str = ""
-
-    def __repr__(self):
-        return (
-            f"AdditionalLayerDescription(data={numpy_repr(self.data)},"
-            f" layer_type='{self.layer_type}', name='{self.name}')"
-        )
-
-
 def dict_repr(dkt: dict) -> str:
     """
     calculate dict representation which use :py:func:`numpy_repr` for numpy representation.
@@ -64,35 +44,61 @@ def dict_repr(dkt: dict) -> str:
 
 
 @dataclass(frozen=True, repr=False)
-class SegmentationResult:
+class ROIExtractionResult:
+    """
+    Class to store results of roi extraction process.
+
+    :ivar np.ndarray roi: Region of Interest represented as numpy array.
+    :ivar ROIExtractionProfile parameters: parameters of roi extraction process.
+    :ivar Dict[str,AdditionalLayerDescription] ~.additional_layers: additional layers returned by algorithm.
+        Could be previewer using napari viewer.
+    :ivar dict roi_annotation: Annotation for ROI. Currently displayed as tooltip
+    :ivar Dict[str,np.ndarray] alternative_representation: Arrays with alternative representations of ROI.
+    :ivar Optional[str] ~.file_path: information on which file roi extraction was performed.
+    """
+
+    # TODO add alternative representation using dict mapping.
     roi: np.ndarray
     parameters: ROIExtractionProfile
     additional_layers: Dict[str, AdditionalLayerDescription] = field(default_factory=dict)
     info_text: str = ""
     roi_annotation: Dict = field(default_factory=dict)
     alternative_representation: Dict[str, np.ndarray] = field(default_factory=dict)
+    file_path: Optional[str] = None
+
+    @property
+    def roi_info(self) -> ROIInfo:
+        return ROIInfo(roi=self.roi, annotations=self.roi_annotation, alternative=self.alternative_representation)
 
     def __post_init__(self):
         if "ROI" in self.alternative_representation:
             raise ValueError("alternative_representation field cannot contain field with ROI key")
+        for key, value in self.additional_layers.items():
+            if value.name == "":
+                value.name = key
 
     def __str__(self):  # pragma: no cover
         return (
-            f"SegmentationResult(segmentation=[shape: {self.roi.shape}, dtype: {self.roi.dtype},"
+            f"SegmentationResult(roi=[shape: {self.roi.shape}, dtype: {self.roi.dtype},"
             f" max: {np.max(self.roi)}], parameters={self.parameters},"
             f" additional_layers={list(self.additional_layers.keys())}, info_text={self.info_text},"
-            f" alternative={dict_repr(self.alternative_representation)}, annotation={dict_repr(self.roi_annotation)}"
+            f" alternative={dict_repr(self.alternative_representation)},"
+            f" roi_annotation={dict_repr(self.roi_annotation)}"
         )
 
     def __repr__(self):  # pragma: no cover
         return (
-            f"SegmentationResult(segmentation=[shape: {self.roi.shape}, dtype: {self.roi.dtype}, "
+            f"SegmentationResult(roi=[shape: {self.roi.shape}, dtype: {self.roi.dtype}, "
             f"max: {np.max(self.roi)}], parameters={self.parameters}, "
             f"additional_layers={list(self.additional_layers.keys())}, info_text={self.info_text},"
-            f" alternative={dict_repr(self.alternative_representation)}, annotation={dict_repr(self.roi_annotation)}"
+            f" alternative={dict_repr(self.alternative_representation)},"
+            f" roi_annotation={dict_repr(self.roi_annotation)}"
         )
 
 
+SegmentationResult = ROIExtractionResult
+
+
 def report_empty_fun(_x, _y):  # pragma: no cover
     pass
 
@@ -147,8 +153,11 @@ class SegmentationAlgorithm(AlgorithmDescribeBase, ABC):
         return self._mask
 
     @mask.setter
-    def mask(self, val: np.ndarray):
-        self._mask = val
+    def mask(self, val: Optional[np.ndarray]):
+        if val is None:
+            self._mask = None
+            return
+        self._mask = self.image.fit_mask_to_image(val)
 
     @classmethod
     @abstractmethod
@@ -164,7 +173,7 @@ class SegmentationAlgorithm(AlgorithmDescribeBase, ABC):
         """Set mask which will limit segmentation area"""
         self.mask = mask
 
-    def calculation_run_wrap(self, report_fun: Callable[[str, int], None]) -> SegmentationResult:
+    def calculation_run_wrap(self, report_fun: Callable[[str, int], None]) -> ROIExtractionResult:
         try:
             return self.calculation_run(report_fun)
         except SegmentationLimitException:  # pragma: no cover
@@ -175,11 +184,11 @@ class SegmentationAlgorithm(AlgorithmDescribeBase, ABC):
             raise SegmentationException(self.get_name(), parameters, image)
 
     @abstractmethod
-    def calculation_run(self, report_fun: Callable[[str, int], None]) -> SegmentationResult:
+    def calculation_run(self, report_fun: Callable[[str, int], None]) -> ROIExtractionResult:
         raise NotImplementedError()
 
     @classmethod
-    def segment_project(cls, project: ProjectInfoBase, parameters: dict) -> SegmentationResult:
+    def segment_project(cls, project: ProjectInfoBase, parameters: dict) -> ROIExtractionResult:
         """
 
         :param ProjectInfoBase project:
@@ -211,6 +220,7 @@ class SegmentationAlgorithm(AlgorithmDescribeBase, ABC):
     def set_image(self, image):
         self.image = image
         self.channel = None
+        self._mask = None
 
     def set_parameters(self, **kwargs):
         base_names = [x.name for x in self.get_fields() if isinstance(x, AlgorithmProperty)]
