@@ -14,14 +14,10 @@ from ..algorithm_describe_base import AlgorithmDescribeBase, AlgorithmProperty,
 from ..channel_class import Channel
 from ..mask_partition_utils import BorderRim as BorderRimBase
 from ..mask_partition_utils import MaskDistanceSplit as MaskDistanceSplitBase
+from ..project_info import AdditionalLayerDescription
 from ..universal_const import Units
 from ..utils import bisect
-from .algorithm_base import (
-    AdditionalLayerDescription,
-    SegmentationAlgorithm,
-    SegmentationLimitException,
-    SegmentationResult,
-)
+from .algorithm_base import ROIExtractionResult, SegmentationAlgorithm, SegmentationLimitException
 from .mu_mid_point import BaseMuMid, mu_mid_dict
 from .noise_filtering import noise_filtering_dict
 from .threshold import BaseThreshold, double_threshold_dict, threshold_dict
@@ -93,10 +89,10 @@ class BorderRim(RestartableAlgorithm):
             return "Need mask"
         return ""
 
-    def calculation_run(self, _report_fun) -> SegmentationResult:
+    def calculation_run(self, _report_fun) -> ROIExtractionResult:
         if self.mask is not None:
             result = BorderRimBase.border_mask(mask=self.mask, voxel_size=self.image.spacing, **self.new_parameters)
-            return SegmentationResult(roi=result, parameters=self.get_segmentation_profile())
+            return ROIExtractionResult(roi=result, parameters=self.get_segmentation_profile())
         raise SegmentationLimitException("Border Rim needs mask")
 
 
@@ -106,12 +102,12 @@ class MaskDistanceSplit(RestartableAlgorithm):
     class in segmentation algorithm interface. It allow user to check how split look with given set of parameters
     """
 
-    def calculation_run(self, report_fun: typing.Callable[[str, int], None]) -> SegmentationResult:
+    def calculation_run(self, report_fun: typing.Callable[[str, int], None]) -> ROIExtractionResult:
         if self.mask is not None:
             result = MaskDistanceSplitBase.split(
                 mask=self.mask, voxel_size=self.image.voxel_size, **self.new_parameters
             )
-            return SegmentationResult(roi=result, parameters=self.get_segmentation_profile())
+            return ROIExtractionResult(roi=result, parameters=self.get_segmentation_profile())
         raise SegmentationLimitException("Mask Distance Split needs mask")
 
     @classmethod
@@ -134,13 +130,13 @@ class ThresholdBaseAlgorithm(RestartableAlgorithm, ABC):
     @classmethod
     def get_fields(cls):
         return [
-            AlgorithmProperty("channel", "Channel", 0, property_type=Channel),
+            AlgorithmProperty("channel", "Channel", 0, value_type=Channel),
             AlgorithmProperty(
                 "noise_filtering",
                 "Filter",
                 next(iter(noise_filtering_dict.keys())),
                 possible_values=noise_filtering_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
             AlgorithmProperty("minimum_size", "Minimum size (px)", 8000, (0, 10 ** 6), 1000),
             AlgorithmProperty(
@@ -153,7 +149,7 @@ class ThresholdBaseAlgorithm(RestartableAlgorithm, ABC):
         ]
 
     def __init__(self, **kwargs):
-        super(ThresholdBaseAlgorithm, self).__init__()
+        super().__init__()
         self.cleaned_image = None
         self.threshold_image = None
         self._sizes_array = []
@@ -178,7 +174,7 @@ class ThresholdBaseAlgorithm(RestartableAlgorithm, ABC):
             "no size filtering": AdditionalLayerDescription(data=full_segmentation, layer_type="labels"),
         }
 
-    def prepare_result(self, roi: np.ndarray) -> SegmentationResult:
+    def prepare_result(self, roi: np.ndarray) -> ROIExtractionResult:
         """
         Collect data for result.
 
@@ -187,7 +183,7 @@ class ThresholdBaseAlgorithm(RestartableAlgorithm, ABC):
         """
         sizes = np.bincount(roi.flat)
         annotation = {i: {"voxels": size} for i, size in enumerate(sizes[1:], 1) if size > 0}
-        return SegmentationResult(
+        return ROIExtractionResult(
             roi=roi,
             parameters=self.get_segmentation_profile(),
             additional_layers=self.get_additional_layers(),
@@ -203,7 +199,7 @@ class ThresholdBaseAlgorithm(RestartableAlgorithm, ABC):
             map(str, self._sizes_array[1 : self.components_num + 1])
         )
 
-    def calculation_run(self, report_fun: typing.Callable[[str, int], typing.Any]) -> SegmentationResult:
+    def calculation_run(self, report_fun: typing.Callable[[str, int], typing.Any]) -> ROIExtractionResult:
         """
         main calculation function
 
@@ -223,18 +219,21 @@ class ThresholdBaseAlgorithm(RestartableAlgorithm, ABC):
             )
             restarted = True
         if restarted or self.new_parameters["threshold"] != self.parameters["threshold"]:
+            if self.parameters["threshold"] is None:
+                restarted = True
             self.parameters["threshold"] = deepcopy(self.new_parameters["threshold"])
             self.threshold_image = self._threshold(self.cleaned_image)
-            if isinstance(self.threshold_info, (list, tuple)):
-                if self.old_threshold_info is None or self.old_threshold_info[0] != self.threshold_info[0]:
-                    restarted = True
+            if isinstance(self.threshold_info, (list, tuple)) and (
+                self.old_threshold_info is None or self.old_threshold_info[0] != self.threshold_info[0]
+            ):
+                restarted = True
             elif self.old_threshold_info != self.threshold_info:
                 restarted = True
             if self.threshold_image.max() == 0:
                 res = self.prepare_result(self.threshold_image.astype(np.uint8))
                 info_text = (
                     "Something wrong with chosen threshold. Please check it. "
-                    "May be to low or to high. The channel bright range is "
+                    "May be too low or too high. The channel brightness range is "
                     f"{self.cleaned_image.min()}-{self.cleaned_image.max()} "
                     f"and chosen threshold is {self.threshold_info}"
                 )
@@ -292,7 +291,7 @@ class OneThresholdAlgorithm(ThresholdBaseAlgorithm, ABC):
                 "Threshold",
                 next(iter(threshold_dict.keys())),
                 possible_values=threshold_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
         )
         return fields
@@ -393,7 +392,7 @@ class BaseThresholdFlowAlgorithm(TwoLevelThresholdBaseAlgorithm, ABC):
                 "Threshold",
                 next(iter(double_threshold_dict.keys())),
                 possible_values=double_threshold_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
         )
         fields.insert(
@@ -403,7 +402,7 @@ class BaseThresholdFlowAlgorithm(TwoLevelThresholdBaseAlgorithm, ABC):
                 "Flow type",
                 next(iter(sprawl_dict.keys())),
                 possible_values=sprawl_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
         )
         for i, el in enumerate(fields):
@@ -439,7 +438,7 @@ class BaseThresholdFlowAlgorithm(TwoLevelThresholdBaseAlgorithm, ABC):
         super().set_image(image)
         self.threshold_info = [None, None]
 
-    def calculation_run(self, report_fun) -> SegmentationResult:
+    def calculation_run(self, report_fun) -> ROIExtractionResult:
         segment_data = super().calculation_run(report_fun)
         if segment_data is not None and self.components_num == 0:
             self.final_sizes = []
@@ -465,7 +464,7 @@ class BaseThresholdFlowAlgorithm(TwoLevelThresholdBaseAlgorithm, ABC):
             self.parameters["sprawl_type"] = self.new_parameters["sprawl_type"]
             new_segment = path_sprawl.sprawl(
                 self.sprawl_area,
-                finally_segment,
+                np.copy(finally_segment),  # TODO add tests for discover this problem
                 self.channel,
                 self.components_num,
                 self.image.spacing,
@@ -476,7 +475,7 @@ class BaseThresholdFlowAlgorithm(TwoLevelThresholdBaseAlgorithm, ABC):
                 self.threshold_info[0],
             )
             self.final_sizes = np.bincount(new_segment.flat)
-            return SegmentationResult(
+            return ROIExtractionResult(
                 roi=new_segment,
                 parameters=self.get_segmentation_profile(),
                 additional_layers=self.get_additional_layers(full_segmentation=self.sprawl_area),
@@ -511,13 +510,13 @@ class OtsuSegment(RestartableAlgorithm):
     @classmethod
     def get_fields(cls):
         return [
-            AlgorithmProperty("channel", "Channel", 0, property_type=Channel),
+            AlgorithmProperty("channel", "Channel", 0, value_type=Channel),
             AlgorithmProperty(
                 "noise_filtering",
                 "Noise Removal",
                 next(iter(noise_filtering_dict.keys())),
                 possible_values=noise_filtering_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
             AlgorithmProperty("components", "Number of Components", 2, (0, 100)),
             # AlgorithmProperty("mask", "Use mask in calculation", True),
@@ -555,7 +554,7 @@ class OtsuSegment(RestartableAlgorithm):
                 self.threshold_info.append(self.threshold_info[-1])
             else:
                 self.threshold_info.append(0)
-        return SegmentationResult(
+        return ROIExtractionResult(
             roi=res,
             parameters=self.get_segmentation_profile(),
             additional_layers={"denoised_image": AdditionalLayerDescription(data=cleaned_image, layer_type="image")},
@@ -579,16 +578,16 @@ class BaseMultiScaleOpening(TwoLevelThresholdBaseAlgorithm, ABC):  # pragma: no
                 "Threshold",
                 next(iter(double_threshold_dict.keys())),
                 possible_values=double_threshold_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
             AlgorithmProperty(
                 "mu_mid",
                 "Mu mid value",
                 next(iter(mu_mid_dict.keys())),
                 possible_values=mu_mid_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
-            AlgorithmProperty("step_limits", "Limits of Steps", 100, options_range=(1, 1000), property_type=int),
+            AlgorithmProperty("step_limits", "Limits of Steps", 100, options_range=(1, 1000), value_type=int),
         ] + super().get_fields()
 
     def get_info_text(self):
@@ -621,7 +620,7 @@ class BaseMultiScaleOpening(TwoLevelThresholdBaseAlgorithm, ABC):  # pragma: no
         super().set_image(image)
         self.threshold_info = [None, None]
 
-    def calculation_run(self, report_fun) -> SegmentationResult:
+    def calculation_run(self, report_fun) -> ROIExtractionResult:
         if self.new_parameters["side_connection"] != self.parameters["side_connection"]:
             neigh, dist = calculate_distances_array(
                 self.image.spacing, get_neigh(self.new_parameters["side_connection"])
