@@ -11,7 +11,8 @@ from PartSegCore.segmentation.watershed import BaseWatershed, sprawl_dict
 from ..algorithm_describe_base import AlgorithmDescribeBase, AlgorithmProperty, ROIExtractionProfile
 from ..channel_class import Channel
 from ..convex_fill import convex_fill
-from ..segmentation.algorithm_base import AdditionalLayerDescription, SegmentationAlgorithm, SegmentationResult
+from ..project_info import AdditionalLayerDescription
+from ..segmentation.algorithm_base import ROIExtractionResult, SegmentationAlgorithm
 from ..utils import bisect
 from .noise_filtering import noise_filtering_dict
 from .threshold import BaseThreshold, double_threshold_dict, threshold_dict
@@ -35,13 +36,13 @@ class ThresholdPreview(StackAlgorithm):
     @classmethod
     def get_fields(cls):
         return [
-            AlgorithmProperty("channel", "Channel", 0, property_type=Channel),
+            AlgorithmProperty("channel", "Channel", 0, value_type=Channel),
             AlgorithmProperty(
                 "noise_filtering",
                 "Filter",
-                next(iter(noise_filtering_dict.keys())),
+                noise_filtering_dict.get_default(),
                 possible_values=noise_filtering_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
             AlgorithmProperty("threshold", "Threshold", 1000, (0, 10 ** 6), 100),
         ]
@@ -51,11 +52,11 @@ class ThresholdPreview(StackAlgorithm):
         return "Only Threshold"
 
     def __init__(self):
-        super(ThresholdPreview, self).__init__()
+        super().__init__()
         self.noise_filtering = None
         self.threshold = 0
 
-    def calculation_run(self, report_fun) -> SegmentationResult:
+    def calculation_run(self, report_fun) -> ROIExtractionResult:
         self.channel = self.get_channel(self.channel_num)
         image = noise_filtering_dict[self.noise_filtering["name"]].noise_filter(
             self.channel, self.image.spacing, self.noise_filtering["values"]
@@ -65,7 +66,7 @@ class ThresholdPreview(StackAlgorithm):
             res[self.mask == 0] = 0
         self.image = None
         self.channel = None
-        return SegmentationResult(
+        return ROIExtractionResult(
             roi=res,
             parameters=self.get_segmentation_profile(),
             additional_layers={"denoised image": AdditionalLayerDescription(layer_type="image", data=image)},
@@ -91,29 +92,29 @@ class BaseThresholdAlgorithm(StackAlgorithm, ABC):
     @classmethod
     def get_fields(cls):
         return [
-            AlgorithmProperty("channel", "Channel", 0, property_type=Channel),
+            AlgorithmProperty("channel", "Channel", 0, value_type=Channel),
             AlgorithmProperty(
                 "noise_filtering",
                 "Filter",
-                next(iter(noise_filtering_dict.keys())),
+                noise_filtering_dict.get_default(),
                 possible_values=noise_filtering_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
             AlgorithmProperty(
                 "threshold",
                 "Threshold",
-                next(iter(threshold_dict.keys())),
+                threshold_dict.get_default(),
                 possible_values=threshold_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
             AlgorithmProperty("close_holes", "Fill holes", True, (True, False)),
             AlgorithmProperty("close_holes_size", "Maximum holes size (px)", 200, (0, 10 ** 5), 10),
             AlgorithmProperty(
                 "smooth_border",
                 "Smooth borders",
-                next(iter(smooth_dict.keys())),
+                smooth_dict.get_default(),
                 possible_values=smooth_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
             AlgorithmProperty(
                 "side_connection",
@@ -136,7 +137,7 @@ class BaseSingleThresholdAlgorithm(BaseThresholdAlgorithm, ABC):
         self.noise_filtering = None
         self.close_holes = False
         self.close_holes_size = 0
-        self.smooth_border = dict()
+        self.smooth_border = {}
         self.gauss_2d = False
         self.edge_connection = True
         self.use_convex = False
@@ -179,7 +180,7 @@ class BaseSingleThresholdAlgorithm(BaseThresholdAlgorithm, ABC):
             report_fun("convex hull", 6)
             resp = convex_fill(resp)
         report_fun("Calculation done", 7)
-        return SegmentationResult(
+        return ROIExtractionResult(
             roi=self.image.fit_array_to_image(resp),
             parameters=self.get_segmentation_profile(),
             additional_layers={
@@ -263,29 +264,29 @@ class ThresholdFlowAlgorithm(BaseThresholdAlgorithm):
     @classmethod
     def get_fields(cls):
         return [
-            AlgorithmProperty("channel", "Channel", 0, property_type=Channel),
+            AlgorithmProperty("channel", "Channel", 0, value_type=Channel),
             AlgorithmProperty(
                 "noise_filtering",
                 "Filter",
-                next(iter(noise_filtering_dict.keys())),
+                noise_filtering_dict.get_default(),
                 possible_values=noise_filtering_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
             AlgorithmProperty(
                 "threshold",
                 "Threshold",
-                next(iter(double_threshold_dict.keys())),
+                double_threshold_dict.get_default(),
                 possible_values=double_threshold_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
             AlgorithmProperty("close_holes", "Fill holes", True, (True, False)),
             AlgorithmProperty("close_holes_size", "Maximum holes size (px)", 200, (0, 10 ** 5), 10),
             AlgorithmProperty(
                 "smooth_border",
                 "Smooth borders",
-                next(iter(smooth_dict.keys())),
+                smooth_dict.get_default(),
                 possible_values=smooth_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
             AlgorithmProperty(
                 "side_connection",
@@ -298,14 +299,14 @@ class ThresholdFlowAlgorithm(BaseThresholdAlgorithm):
             AlgorithmProperty(
                 "sprawl_type",
                 "Flow type",
-                next(iter(sprawl_dict.keys())),
+                sprawl_dict.get_default(),
                 possible_values=sprawl_dict,
-                property_type=AlgorithmDescribeBase,
+                value_type=AlgorithmDescribeBase,
             ),
             AlgorithmProperty("use_convex", "Use convex hull", False, (True, False)),
         ]
 
-    def calculation_run(self, report_fun: Callable[[str, int], None]) -> SegmentationResult:
+    def calculation_run(self, report_fun: Callable[[str, int], None]) -> ROIExtractionResult:
         report_fun("Noise removal", 0)
         self.channel = self.get_channel(self.channel_num)
         noise_filtered = noise_filtering_dict[self.parameters["noise_filtering"]["name"]].noise_filter(
@@ -356,7 +357,7 @@ class ThresholdFlowAlgorithm(BaseThresholdAlgorithm):
             report_fun("convex hull", 6)
             segmentation = convex_fill(segmentation)
         report_fun("Calculation done", 7)
-        return SegmentationResult(
+        return ROIExtractionResult(
             roi=segmentation,
             parameters=self.get_segmentation_profile(),
             additional_layers={
@@ -416,8 +417,7 @@ class AutoThresholdAlgorithm(BaseSingleThresholdAlgorithm):
             report_fun("Components exclusion apply", 1)
             image[self.mask == 0] = 0
         report_fun("Threshold calculation", 2)
-        mask = self._threshold_image(image)
-        return mask
+        return self._threshold_image(image)
 
     def set_parameters(self, suggested_size, *args, **kwargs):  # pylint: disable=W0221
         self._set_parameters(*args, **kwargs)
@@ -436,7 +436,7 @@ final_algorithm_list = [ThresholdAlgorithm, ThresholdFlowAlgorithm, ThresholdPre
 
 
 def close_small_holes(image, max_hole_size):
-    if image.dtype == np.bool:
+    if image.dtype == bool:
         image = image.astype(np.uint8)
     if len(image.shape) == 2:
         rev_conn = sitk.ConnectedComponent(sitk.BinaryNot(sitk.GetImageFromArray(image)), True)
