@@ -78,10 +78,7 @@ class PathWatershed(BaseWatershed):
         lower_bound,
         upper_bound,
     ):
-        if operator(1, 0):
-            path_sprawl = path_maximum_sprawl
-        else:
-            path_sprawl = path_minimum_sprawl
+        path_sprawl = path_maximum_sprawl if operator(1, 0) else path_minimum_sprawl
         # print(path_sprawl)
         image = data.astype(np.float64)
         image[sprawl_area == 0] = 0
@@ -193,8 +190,8 @@ class MSOWatershed(BaseWatershed):
     @classmethod
     def get_fields(cls):
         return [
-            AlgorithmProperty("step_limits", "Limits of Steps", 100, options_range=(1, 1000), property_type=int),
-            AlgorithmProperty("reflective", "Reflective", False, property_type=bool),
+            AlgorithmProperty("step_limits", "Limits of Steps", 100, options_range=(1, 1000), value_type=int),
+            AlgorithmProperty("reflective", "Reflective", False, value_type=bool),
         ]
 
     @classmethod
@@ -228,7 +225,13 @@ class MSOWatershed(BaseWatershed):
         if arguments["reflective"]:
             mu_array[mu_array < 0.5] = 1 - mu_array[mu_array < 0.5]
         mso.set_mu_array(mu_array)
-        mso.run_MSO(arguments["step_limits"])
+        try:
+            mso.run_MSO(arguments["step_limits"])
+        except RuntimeError as e:
+            if e.args[0] == "to many steps: constrained dilation":
+                raise SegmentationLimitException(*e.args)
+            raise
+
         # print("Steps: ", mso.steps_done(), file=sys.stderr)
         result = mso.get_result_catted()
         result[result > 0] -= 1
