@@ -1,11 +1,17 @@
+import re
 import typing
 import warnings
+from collections.abc import Iterable
+from itertools import zip_longest
 
 import numpy as np
 
 Spacing = typing.Tuple[typing.Union[float, int], ...]
 
 _DEF = object()
+FRAME_THICKNESS = 2
+
+DEFAULT_SCALE_FACTOR = 10 ** 9
 
 
 def minimal_dtype(val: int):
@@ -48,7 +54,8 @@ def reduce_array(
     if max_val is None:
         max_val = np.max(array)
 
-    translate = np.zeros(max_val + 1, dtype=dtype if dtype else minimal_dtype(len(components) + 1))
+    translate = np.zeros(max_val + 1, dtype=dtype or minimal_dtype(len(components) + 1))
+
     for i, val in enumerate(sorted(components), start=0 if 0 in components else 1):
         translate[val] = i
 
@@ -65,7 +72,7 @@ class Image:
     :param mask: mask array in shape z,y,x
     :param default_coloring: default colormap - not used yet
     :param ranges: default ranges for channels
-    :param labels: labels for channels
+    :param channel_names: labels for channels
     :param axes_order: allow to create Image object form data with different axes order, or missed axes
 
     :cvar str ~.axis_order: internal order of axes
@@ -84,7 +91,7 @@ class Image:
         if hasattr(cls, "return_order"):  # pragma: no cover
             warnings.warn("Using return_order is deprecated since PartSeg 0.11.0", DeprecationWarning)
             cls.axis_order = cls.return_order
-        return super(Image, cls).__new__(cls)
+        return super().__new__(cls)
 
     def __init__(
         self,
@@ -94,7 +101,7 @@ class Image:
         mask: typing.Union[None, np.ndarray] = None,
         default_coloring=None,
         ranges=None,
-        labels=None,
+        channel_names=None,
         axes_order: typing.Optional[str] = None,
     ):
         # TODO add time distance to image spacing
@@ -102,20 +109,27 @@ class Image:
             axes_order = self.axis_order
         if data.ndim != len(axes_order):
             raise ValueError(
-                "Data should have same number of dimensions " f"like length of axes_order (current :{len(axes_order)}"
+                "Data should have same number of dimensions "
+                f"like length of axes_order (axis :{len(axes_order)}, ndim: {data.ndim}"
             )
         if not isinstance(image_spacing, tuple):
             image_spacing = tuple(image_spacing)
         self._image_array = self.reorder_axes(data, axes_order)
         self._image_spacing = (1.0,) * (3 - len(image_spacing)) + image_spacing
-        self._image_spacing = tuple([el if el > 0 else 10 ** -6 for el in self._image_spacing])
+        self._image_spacing = tuple(el if el > 0 else 10 ** -6 for el in self._image_spacing)
+
         self.file_path = file_path
         self.default_coloring = default_coloring
         if self.default_coloring is not None:
             self.default_coloring = [np.array(x) for x in default_coloring]
-        self.labels = labels
-        if isinstance(self.labels, (tuple, list)):
-            self.labels = self.labels[: self.channels]
+        default_channel_names = [f"channel {i+1}" for i in range(self.channels)]
+        if isinstance(channel_names, Iterable):
+            self._channel_names = [
+                x if x is not None else y for x, y in zip_longest(channel_names, default_channel_names, fillvalue=None)
+            ]
+        else:
+            self._channel_names = default_channel_names
+        self._channel_names = self._channel_names[: self.channels]
         if ranges is None:
             axis = list(range(len(self.axis_order)))
             axis.remove(self.axis_order.index("C"))
@@ -149,7 +163,27 @@ class Image:
             axis = self.axis_order.index(axis)
         data = self.reorder_axes(image.get_data(), image.axis_order)
         data = np.concatenate((self.get_data(), data), axis=axis)
-        return self.substitute(data=data, ranges=self.ranges + image.ranges)
+        channel_names = self.channel_names
+        reg = re.compile(r"channel \d+")
+        for name in image.channel_names:
+            match = reg.match(name)
+            new_name = name
+            if match:
+                name = "channel"
+                new_name = f"channel {len(channel_names) + 1}"
+            i = 1
+            while new_name in channel_names:
+                new_name = f"{name} ({i})"
+                i += 1
+                if i > 10000:  # pragma: no cover
+                    raise ValueError("fail when try to fix channel name")
+            channel_names.append(new_name)
+
+        return self.substitute(data=data, ranges=self.ranges + image.ranges, channel_names=channel_names)
+
+    @property
+    def channel_names(self) -> typing.List[str]:
+        return self._channel_names[:]
 
     @property
     def channel_pos(self) -> int:
@@ -213,7 +247,7 @@ class Image:
         """
         :return: letters which indicates non trivial dimensions
         """
-        return "".join([key for val, key in zip(self._image_array.shape, self.axis_order) if val > 1])
+        return "".join(key for val, key in zip(self._image_array.shape, self.axis_order) if val > 1)
 
     def substitute(
         self,
@@ -223,7 +257,7 @@ class Image:
         mask=_DEF,
         default_coloring=None,
         ranges=None,
-        labels=None,
+        channel_names=None,
     ):
         """Create copy of image with substitution of not None elements"""
         data = self._image_array if data is None else data
@@ -232,7 +266,7 @@ class Image:
         mask = self._mask_array if mask is _DEF else mask
         default_coloring = self.default_coloring if default_coloring is None else default_coloring
         ranges = self.ranges if ranges is None else ranges
-        labels = self.labels if labels is None else labels
+        channel_names = self.channel_names if channel_names is None else channel_names
         return self.__class__(
             data=data,
             image_spacing=image_spacing,
@@ -240,7 +274,7 @@ class Image:
             mask=mask,
             default_coloring=default_coloring,
             ranges=ranges,
-            labels=labels,
+            channel_names=channel_names,
         )
 
     def set_mask(self, mask: typing.Optional[np.ndarray], axes: typing.Optional[str] = None):
@@ -279,7 +313,15 @@ class Image:
         return np.reshape(array, shape)
 
     def fit_array_to_image(self, array: np.ndarray) -> np.ndarray:
-        """change shape of array with inserting singe dimensional entries"""
+        """
+        Change shape of array with inserting singe dimensional entries
+
+        :param np.ndarray array: array to be fitted
+
+        :return: reshaped array witha added missing 1 in shape
+
+        :raises ValueError: if cannot fit array
+        """
         base_shape = list(self._image_array.shape)
         base_shape.pop(self.channel_pos)
         return self._fit_array_to_image(base_shape, array)
@@ -451,7 +493,7 @@ class Image:
             return tuple(self._image_spacing[1:])
         return self._image_spacing
 
-    def normalized_scaling(self, factor=10 ** 9) -> Spacing:
+    def normalized_scaling(self, factor=DEFAULT_SCALE_FACTOR) -> Spacing:
         if self.is_2d:
             return (1, 1) + tuple(np.multiply(self.spacing, factor))
         return (1,) + tuple(np.multiply(self.spacing, factor))
@@ -463,11 +505,11 @@ class Image:
 
     def set_spacing(self, value: Spacing):
         """set image spacing"""
-        if any(x == 0 for x in value):
+        if 0 in value:
             return
         if self.is_2d and len(value) + 1 == len(self._image_spacing):
             value = (1.0,) + tuple(value)
-        if len(value) != len(self._image_spacing):
+        if len(value) != len(self._image_spacing):  # pragma: no cover
             raise ValueError("Correction of spacing fail.")
         self._image_spacing = tuple(value)
 
@@ -479,15 +521,22 @@ class Image:
         image_pos = [slice(None) for _ in range(array.ndim)]
 
         for index in index_to_add:
-            result_shape[index] += 2
-            image_pos[index] = slice(1, result_shape[index] - 1)
+            result_shape[index] += FRAME_THICKNESS * 2
+            image_pos[index] = slice(FRAME_THICKNESS, result_shape[index] - FRAME_THICKNESS)
 
         data = np.zeros(shape=result_shape, dtype=array.dtype)
         data[tuple(image_pos)] = array
         return data
 
     @staticmethod
-    def _calc_index_to_frame(array_axis, important_axis):
+    def calc_index_to_frame(array_axis, important_axis):
+        """
+        calculate in which axis frame should be added
+
+        :param str array_axis: list of image axis
+        :param str important_axis: list of framed axis
+        :return:
+        """
         return [array_axis.index(letter) for letter in important_axis]
 
     def cut_image(
@@ -530,13 +579,13 @@ class Image:
         important_axis = "XY" if self.is_2d else "XYZ"
 
         return self.__class__(
-            self._frame_array(new_image, self._calc_index_to_frame(self.axis_order, important_axis)),
+            self._frame_array(new_image, self.calc_index_to_frame(self.axis_order, important_axis)),
             self._image_spacing,
             None,
-            self._frame_array(new_mask, self._calc_index_to_frame(self.array_axis_order, important_axis)),
+            self._frame_array(new_mask, self.calc_index_to_frame(self.array_axis_order, important_axis)),
             self.default_coloring,
             self.ranges,
-            self.labels,
+            self.channel_names,
         )
 
     def get_imagej_colors(self):
@@ -579,7 +628,7 @@ class Image:
 
     def get_um_spacing(self) -> Spacing:
         """image spacing in micrometers"""
-        return tuple([float(x * 10 ** 6) for x in self.spacing])
+        return tuple(float(x * 10 ** 6) for x in self.spacing)
 
     def get_ranges(self) -> typing.List[typing.Tuple[float, float]]:
         """image brightness ranges for each channel"""
@@ -588,12 +637,12 @@ class Image:
     def __str__(self):
         return (
             f"{self.__class__} Shape {self._image_array.shape}, dtype: {self._image_array.dtype}, "
-            f"labels: {self.labels}, coloring: {self.get_colors()} mask: {self.has_mask}"
+            f"labels: {self.channel_names}, coloring: {self.get_colors()} mask: {self.has_mask}"
         )
 
     def __repr__(self):
         mask_info = f"mask=True, mask_dtype={self._mask_array.dtype}" if self.mask is not None else "mask=False"
         return (
             f"Image(shape={self._image_array.shape} dtype={self._image_array.dtype}, spacing={self.spacing}, "
-            f"labels={self.labels}, channels={self.channels}, axes={repr(self.axis_order)}, {mask_info})"
+            f"labels={self.channel_names}, channels={self.channels}, axes={repr(self.axis_order)}, {mask_info})"
         )
