@@ -9,6 +9,7 @@ from threading import Lock
 import numpy as np
 import tifffile.tifffile
 from czifile.czifile import CziFile
+from defusedxml import ElementTree
 from oiffile import OifFile
 from tifffile import TiffFile
 
@@ -133,7 +134,7 @@ class BaseImageReader:
             final_mapping = [final_mapping_dict[letter] for letter in axes]
         except KeyError as e:  # pragma: no cover
             raise NotImplementedError(
-                f"Data type not supported ({e.args[0]})." f" Please contact with author for update code"
+                f"Data type not supported ({e.args[0]}). Please contact with author for update code"
             )
         if len(final_mapping) != len(set(final_mapping)):
             raise NotImplementedError("Data type not supported. Please contact with author for update code")
@@ -145,11 +146,11 @@ class BaseImageReader:
 
 
 class GenericImageReader(BaseImageReader):
-    """This class try to decide which method use base on path """
+    """This class try to decide which method use base on path"""
 
     def read(self, image_path: typing.Union[str, BytesIO, Path], mask_path=None, ext=None) -> Image:
         if ext is None:
-            if isinstance(image_path, str):
+            if isinstance(image_path, (str, Path)):
                 ext = os.path.splitext(image_path)[1]
             else:
                 ext = ".tif"
@@ -158,6 +159,8 @@ class GenericImageReader(BaseImageReader):
             return CziImageReader.read_image(image_path, mask_path, self.callback_function, self.default_spacing)
         if ext in [".oif", ".oib"]:
             return OifImagReader.read_image(image_path, mask_path, self.callback_function, self.default_spacing)
+        if ext == ".obsep":
+            return ObsepImageReader.read_image(image_path, mask_path, self.callback_function, self.default_spacing)
         return TiffImageReader.read_image(image_path, mask_path, self.callback_function, self.default_spacing)
 
 
@@ -206,9 +209,7 @@ class CziImageReader(BaseImageReader):
         metadata = self.image_file.metadata(False)
         try:
             scaling = metadata["ImageDocument"]["Metadata"]["Scaling"]["Items"]["Distance"]
-            scale_info = {}
-            for el in scaling:
-                scale_info[el["Id"]] = el["Value"]
+            scale_info = {el["Id"]: el["Value"] for el in scaling}
             self.spacing = (
                 scale_info.get("Z", self.default_spacing[0]),
                 scale_info.get("Y", self.default_spacing[1]),
@@ -238,6 +239,52 @@ class CziImageReader(BaseImageReader):
         return super().update_array_shape(array, axes)
 
 
+class ObsepImageReader(BaseImageReader):
+    def read(self, image_path: typing.Union[str, BytesIO, Path], mask_path=None, ext=None) -> Image:
+        directory = Path(os.path.dirname(image_path))
+        xml_doc = ElementTree.parse(image_path).getroot()
+        channels = xml_doc.findall("net/node/node/attribute[@name='image type']")
+        if not channels:
+            raise ValueError("Information about channel images not found")
+        possible_extensions = [".tiff", ".tif", ".TIFF", ".TIF"]
+        channel_list = []
+        for channel in channels:
+            try:
+                name = next(iter(channel)).attrib["val"]
+            except StopIteration:  # pragma: no cover
+                raise ValueError("Missed information about channel name in obsep file")
+            for ex in possible_extensions:
+                if (directory / (name + ex)).exists():
+                    name += ex
+                    break
+            else:  # pragma: no cover
+                raise ValueError(f"Not found file for key {name}")
+            channel_list.append(TiffImageReader.read_image(directory / name, default_spacing=self.default_spacing))
+        for channel in channels:
+            try:
+                name = next(iter(channel)).attrib["val"] + "_deconv"
+            except StopIteration:  # pragma: no cover
+                raise ValueError("Missed information about channel name in obsep file")
+            for ex in possible_extensions:
+                if (directory / (name + ex)).exists():
+                    name += ex
+                    break
+            if (directory / name).exists():
+                channel_list.append(TiffImageReader.read_image(directory / name, default_spacing=self.default_spacing))
+
+        image = channel_list[0]
+        for el in channel_list[1:]:
+            image = image.merge(el, "C")
+
+        z_spacing = (
+            float(xml_doc.find("net/node/attribute[@name='step width']/double").attrib["val"]) * name_to_scalar["um"]
+        )
+
+        image.set_spacing((z_spacing,) + image.spacing[1:])
+        image.file_path = str(image_path)
+        return image
+
+
 class TiffImageReader(BaseImageReader):
     """
     TIFF/LSM files reader. Base reading with :py:meth:`BaseImageReader.read_image`
@@ -251,14 +298,14 @@ class TiffImageReader(BaseImageReader):
         self.image_file = None
         self.mask_file: typing.Optional[TiffFile] = None
         self.colors = None
-        self.labels = None
+        self.channel_names = None
         self.ranges = None
 
     def read(self, image_path: typing.Union[str, BytesIO, Path], mask_path=None, ext=None) -> Image:
         """
         Read tiff image from tiff_file
         """
-        self.spacing, self.colors, self.labels, self.ranges = self.default_spacing, None, None, None
+        self.spacing, self.colors, self.channel_names, self.ranges = self.default_spacing, None, None, None
         self.image_file = TiffFile(image_path)
         total_pages_num = len(self.image_file.series[0])
         if mask_path is not None:
@@ -305,14 +352,14 @@ class TiffImageReader(BaseImageReader):
         self.image_file.close()
         if self.mask_file is not None:
             self.mask_file.close()
-        if not isinstance(image_path, str):
+        if not isinstance(image_path, (str, Path)):
             image_path = ""
         return self.image_class(
             image_data,
             self.spacing,
             mask=mask_data,
             default_coloring=self.colors,
-            labels=self.labels,
+            channel_names=self.channel_names,
             ranges=self.ranges,
             file_path=os.path.abspath(image_path),
             axes_order=self.return_order(),
@@ -332,10 +379,10 @@ class TiffImageReader(BaseImageReader):
                 continue
             try:
                 j = image_series.axes.index(pos)
-            except ValueError:
+            except ValueError:  # pragma: no cover
                 raise ValueError("Incompatible shape of mask and image (axes)")
                 # TODO add verification if problem with T/Z/I
-            if image_series.shape[j] != mask_series.shape[i]:
+            if image_series.shape[j] != mask_series.shape[i]:  # pragma: no cover
                 raise ValueError("Incompatible shape of mask and image")
             # TODO Add verification if mask have to few dimensions
 
@@ -361,7 +408,7 @@ class TiffImageReader(BaseImageReader):
                     scalar = name_to_scalar["centimeter"]
                 elif unit == 2:
                     scalar = name_to_scalar["cal"]
-                else:
+                else:  # pragma: no cover
                     raise KeyError(f"wrong scalar {tags['ResolutionUnit']}, {tags['ResolutionUnit'].value}")
 
             x_spacing = tags["XResolution"].value[1] / tags["XResolution"].value[0] * scalar
@@ -380,7 +427,7 @@ class TiffImageReader(BaseImageReader):
         x_spacing, y_spacing = self.read_resolution_from_tags()
         self.spacing = z_spacing, y_spacing, x_spacing
         self.colors = self.image_file.imagej_metadata.get("LUTs")
-        self.labels = self.image_file.imagej_metadata.get("Labels")
+        self.channel_names = self.image_file.imagej_metadata.get("Labels")
         if "Ranges" in self.image_file.imagej_metadata:
             ranges = self.image_file.imagej_metadata["Ranges"]
             self.ranges = list(zip(ranges[::2], ranges[1::2]))
@@ -402,7 +449,7 @@ class TiffImageReader(BaseImageReader):
             pass
         if "Channel" in meta_data and isinstance(meta_data["Channel"], (list, tuple)):
             try:
-                self.labels = [ch["Name"] for ch in meta_data["Channel"]]
+                self.channel_names = [ch["Name"] for ch in meta_data["Channel"]]
             except KeyError:
                 pass
             try:
@@ -416,7 +463,7 @@ class TiffImageReader(BaseImageReader):
             if "Colors" in self.image_file.lsm_metadata["ChannelColors"]:
                 self.colors = [x[:3] for x in self.image_file.lsm_metadata["ChannelColors"]["Colors"]]
             if "ColorNames" in self.image_file.lsm_metadata["ChannelColors"]:
-                self.labels = self.image_file.lsm_metadata["ChannelColors"]["ColorNames"]
+                self.channel_names = self.image_file.lsm_metadata["ChannelColors"]["ColorNames"]
 
 
 name_to_scalar = {
