@@ -1,4 +1,5 @@
 import pytest
+from qtpy.QtWidgets import QApplication
 
 from PartSegCore.algorithm_describe_base import ROIExtractionProfile
 from PartSegCore.analysis import SegmentationPipeline, SegmentationPipelineElement
@@ -21,12 +22,12 @@ try:
         return settings
 
     @pytest.fixture
-    def stack_settings(qtbot, tmp_path):
+    def stack_settings(tmp_path):
         settings = StackSettings(tmp_path)
         chose = ChosenComponents()
-        qtbot.addWidget(chose)
         settings.chosen_components_widget = chose
-        return settings
+        yield settings
+        chose.deleteLater()
 
     @pytest.fixture
     def part_settings_with_project(image, analysis_segmentation2, tmp_path):
@@ -63,6 +64,60 @@ try:
 
         monkeypatch.setattr(napari_image_view.ImageView, "_add_layer_util", _add_layer_util)
 
+    @pytest.fixture(autouse=True)
+    def check_opened_windows(qapp):
+        yield
+        widgets = qapp.topLevelWidgets()
+        for widget in widgets:
+            assert not widget.isVisible()
+
+    @pytest.fixture(autouse=True)
+    def block_threads(monkeypatch, request):
+        if "enablethread" in request.keywords:
+            return
+
+        from pytestqt.qt_compat import qt_api
+        from qtpy.QtCore import QThread, QTimer
+
+        old_start = QTimer.start
+
+        class OldTimer(QTimer):
+            def start(self, time=None):
+                if time is not None:
+                    old_start(self, time)
+                else:
+                    old_start(self)
+
+        def not_start(self):
+            raise RuntimeError("Thread should not be used in test")
+
+        monkeypatch.setattr(QTimer, "start", not_start)
+        monkeypatch.setattr(QThread, "start", not_start)
+        monkeypatch.setattr(qt_api.QtCore, "QTimer", OldTimer)
+
+    @pytest.fixture(autouse=True)
+    def clean_settings():
+        try:
+            from napari.utils.settings import SETTINGS
+
+            SETTINGS.reset()
+            yield
+            try:
+                SETTINGS.reset()
+            except AttributeError:
+                pass
+        except ImportError:
+            yield
+
+    @pytest.fixture
+    def leaked_widgets():
+        initial = QApplication.topLevelWidgets()
+        yield
+        QApplication.processEvents()
+        leak = set(QApplication.topLevelWidgets()).difference(initial)
+        if any(n.__class__.__name__ != "CanvasBackendDesktop" for n in leak):
+            raise AssertionError(f"Widgets ({len(leak)}) leaked!: {leak}")
+
 
 except (RuntimeError, ImportError):
     pass
