@@ -1,3 +1,5 @@
+# pylint: disable=R0201
+
 import itertools
 import os
 from functools import partial, reduce
@@ -11,12 +13,15 @@ from sympy import symbols
 from PartSegCore.analysis import load_metadata
 from PartSegCore.analysis.measurement_base import AreaType, Leaf, MeasurementEntry, Node, PerComponent
 from PartSegCore.analysis.measurement_calculation import (
+    HARALIC_FEATURES,
+    MEASUREMENT_DICT,
     ComponentsInfo,
     ComponentsNumber,
     Diameter,
     DistanceMaskSegmentation,
     DistancePoint,
     FirstPrincipalAxisLength,
+    Haralick,
     MaximumPixelBrightness,
     MeanPixelBrightness,
     MeasurementProfile,
@@ -559,7 +564,8 @@ class TestRimVolume:
     @pytest.mark.parametrize("image", [get_cube_image(), get_square_image()], ids=["cube", "square"])
     @pytest.mark.parametrize("scale", [1, 4])
     def test_image(self, image, scale):
-        image = image.substitute(image_spacing=tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image = image.substitute(image_spacing=tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
+
         mask1 = image.get_channel(0)[0] > 40
         mask2 = image.get_channel(0)[0] > 60
         mask3 = mask1 * ~mask2
@@ -640,7 +646,7 @@ class TestRimPixelBrightnessSum:
 
     @pytest.mark.parametrize("image", [get_cube_image(), get_square_image()], ids=["cube", "square"])
     def test_image(self, image):
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         mask1 = image.get_channel(0)[0] > 40
         mask2 = image.get_channel(0)[0] > 60
         mask3 = mask1 * ~mask2
@@ -1039,7 +1045,8 @@ class TestSplitOnPartVolume:
         assert leaf.channel is None
 
     def test_cube_equal_radius(self, cube_image):
-        cube_image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in cube_image.spacing]))
+        cube_image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in cube_image.spacing))
+
         mask1 = cube_image.get_channel(0)[0] > 40
         mask2 = cube_image.get_channel(0)[0] > 60
         result_scale = reduce(lambda x, y: x * y, cube_image.voxel_size)
@@ -1150,7 +1157,7 @@ class TestSplitOnPartVolume:
 
     def test_result_scalar(self):
         image = get_cube_image()
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         mask1 = image.get_channel(0)[0] > 40
         mask2 = image.get_channel(0)[0] > 60
         result_scale = reduce(lambda x, y: x * y, image.voxel_size)
@@ -1205,7 +1212,7 @@ class TestSplitOnPartVolume:
 
     def test_square_equal_radius(self):
         image = get_square_image()
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         mask1 = image.get_channel(0)[0] > 40
         mask2 = image.get_channel(0)[0] > 60
 
@@ -1265,7 +1272,7 @@ class TestSplitOnPartVolume:
 
     def test_square_equal_volume(self):
         image = get_square_image()
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         mask1 = image.get_channel(0)[0] > 40
         mask2 = image.get_channel(0)[0] > 60
 
@@ -1362,7 +1369,8 @@ class TestSplitOnPartPixelBrightnessSum:
         ],
     )
     def test_cube_equal_radius(self, cube_image, nr, sum_val, diff_array):
-        cube_image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in cube_image.spacing]))
+        cube_image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in cube_image.spacing))
+
         mask1 = cube_image.get_channel(0)[0] > 40
         mask2 = cube_image.get_channel(0)[0] > 60
 
@@ -1397,7 +1405,7 @@ class TestSplitOnPartPixelBrightnessSum:
         data[0, 10:50, 20:80, 20:80] = 50
         data[0, 15:45, 30:70, 30:70] = 70
         image = Image(data, (100, 50, 50), "")
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         mask1 = image.get_channel(0)[0] > 40
         mask2 = image.get_channel(0)[0] > 60
 
@@ -1429,7 +1437,7 @@ class TestSplitOnPartPixelBrightnessSum:
     )
     def test_square(self, nr, sum_val, diff_array, equal_volume):
         image = get_square_image()
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         mask1 = image.get_channel(0)[0] > 40
         mask2 = image.get_channel(0)[0] > 60
 
@@ -1450,7 +1458,7 @@ class TestSplitOnPartPixelBrightnessSum:
 class TestStatisticProfile:
     def test_cube_volume_area_type(self):
         image = get_cube_image()
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         image.set_mask((image.get_channel(0)[0] > 40).astype(np.uint8))
         segmentation = (image.get_channel(0)[0] > 60).astype(np.uint8)
 
@@ -1480,7 +1488,7 @@ class TestStatisticProfile:
 
     def test_square_volume_area_type(self):
         image = get_square_image()
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         image.set_mask((image.get_channel(0)[0] > 40).astype(np.uint8))
         segmentation = (image.get_channel(0)[0] > 60).astype(np.uint8)
 
@@ -1510,7 +1518,7 @@ class TestStatisticProfile:
 
     def test_cube_pixel_sum_area_type(self):
         image = get_cube_image()
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         image.set_mask((image.get_channel(0)[0] > 40).astype(np.uint8))
         segmentation = (image.get_channel(0)[0] > 60).astype(np.uint8)
 
@@ -1542,7 +1550,7 @@ class TestStatisticProfile:
 
     def test_cube_surface_area_type(self):
         image = get_cube_image()
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         image.set_mask((image.get_channel(0)[0] > 40).astype(np.uint8))
         segmentation = (image.get_channel(0)[0] > 60).astype(np.uint8)
 
@@ -1571,7 +1579,7 @@ class TestStatisticProfile:
 
     def test_cube_density(self):
         image = get_cube_image()
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         image.set_mask((image.get_channel(0)[0] > 40).astype(np.uint8))
         segmentation = (image.get_channel(0)[0] > 60).astype(np.uint8)
 
@@ -1642,7 +1650,7 @@ class TestStatisticProfile:
 
     def test_cube_volume_power(self):
         image = get_cube_image()
-        image.set_spacing(tuple([x / UNIT_SCALE[Units.nm.value] for x in image.spacing]))
+        image.set_spacing(tuple(x / UNIT_SCALE[Units.nm.value] for x in image.spacing))
         image.set_mask((image.get_channel(0)[0] > 40).astype(np.uint8))
         segmentation = (image.get_channel(0)[0] > 60).astype(np.uint8)
 
@@ -1753,7 +1761,7 @@ class TestStatisticProfile:
         assert result["LongestMainAxisLength per component"][0][1] == 26 * 50 * UNIT_SCALE[Units.nm.value]
 
     def test_all_variants(self, bundle_test_dir):
-        """ This test check if all calculations finished, not values. """
+        """This test check if all calculations finished, not values."""
         file_path = os.path.join(bundle_test_dir, "measurements_profile.json")
         assert os.path.exists(file_path)
         profile = load_metadata(file_path)["all_statistic"]
@@ -1954,3 +1962,63 @@ class TestMeasurementResult:
             ["test.tif", 3, 2, 1, 6, 3],
         ]
         assert storage.get_labels() == ["File name", "Segmentation component", "Mask component", "aa", "bb", "cc"]
+
+    def test_to_dataframe(self):
+        info = ComponentsInfo(np.arange(1, 4), np.arange(1, 3), {1: [1], 2: [2], 3: [1, 2]})
+        storage = MeasurementResult(info)
+        storage["aa"] = 1, "", (PerComponent.No, AreaType.ROI)
+        storage["bb"] = [4, 5, 6], "np", (PerComponent.Yes, AreaType.ROI)
+        df = storage.to_dataframe()
+        assert np.all(df.columns == ["aa", "bb (np)"])
+        assert np.all(df.index == [1, 2, 3])
+        assert np.all(df.values == [[1, 4], [1, 5], [1, 6]])
+
+
+class TestHaralick:
+    def test_base(self):
+        data = np.zeros((10, 20, 20), dtype=np.uint8)
+        data[1:-1, 3:-3, 3:-3] = 2
+        data[1:-1, 4:-4, 4:-4] = 3
+        mask = data > 0
+        res = Haralick.calculate_property(mask, data, distance=1, feature=HARALIC_FEATURES[0])
+        assert res.size == 1
+
+    def test_4d_base(self):
+        data = np.zeros((1, 10, 20, 20), dtype=np.uint8)
+        data[0, :-1, 3:-3, 3:-3] = 2
+        data[0, 1:-1, 4:-4, 4:-4] = 3
+        mask = data > 0
+        res = Haralick.calculate_property(mask, data, distance=1, feature=HARALIC_FEATURES[0])
+        assert res.size == 1
+
+    @pytest.mark.parametrize("feature", HARALIC_FEATURES)
+    @pytest.mark.parametrize("distance", range(1, 5))
+    def test_variants(self, feature, distance):
+        data = np.zeros((10, 20, 20), dtype=np.uint8)
+        data[1:-1, 3:-3, 3:-3] = 2
+        data[1:-1, 4:-4, 4:-4] = 3
+        mask = data > 0
+        Haralick.calculate_property(mask, data, distance=distance, feature=feature)
+
+
+@pytest.mark.parametrize("method", MEASUREMENT_DICT.values())
+@pytest.mark.parametrize("dtype", [float, int, np.uint8, np.uint16, np.uint32, np.float16, np.float32])
+def ttest_all_methods(method, dtype):
+    data = np.zeros((10, 20, 20), dtype=dtype)
+    data[1:-1, 3:-3, 3:-3] = 2
+    data[1:-1, 4:-4, 4:-4] = 3
+    roi = (data > 0).astype(np.uint8)
+    mask = (data > 2).astype(np.uint8)
+
+    res = method.calculate_property(
+        area_array=roi,
+        mask=mask,
+        channel=data,
+        channel_num=0,
+        voxel_size=(1, 1, 1),
+        result_scalar=1,
+        roi_alternative={},
+        roi_annotation={},
+        **method.get_default_values(),
+    )
+    float(res)
