@@ -1,3 +1,5 @@
+# pylint: disable=R0201
+
 import operator
 from abc import ABC
 from copy import deepcopy
@@ -623,7 +625,8 @@ class TestPipeline:
         data[0, 10:40, 20:80, 40:80, 1] = 10
         return Image(data, (100, 50, 50), "")
 
-    def test_pipeline_simple(self):
+    @pytest.mark.parametrize("use_mask", [True, False])
+    def test_pipeline_simple(self, use_mask):
         image = self.get_image()
         prop1 = MaskProperty(
             dilate=RadiusType.NO,
@@ -652,10 +655,11 @@ class TestPipeline:
         seg_profile2 = ROIExtractionProfile(name="Unknown", algorithm="Lower threshold", values=parameters2)
 
         pipeline = SegmentationPipeline(name="test", segmentation=seg_profile2, mask_history=[pipeline_element])
-        result = calculate_pipeline(image=image, mask=None, pipeline=pipeline, report_fun=empty)
+        mask = np.ones(image.get_channel(0).shape, dtype=np.uint8) if use_mask else None
+        result = calculate_pipeline(image=image, mask=mask, pipeline=pipeline, report_fun=empty)
         result_segmentation = np.zeros((50, 100, 100), dtype=np.uint8)
         result_segmentation[10:40, 20:80, 40:60] = 1
-        assert np.all(result.roi == result_segmentation)
+        assert np.all(result.roi_info.roi == result_segmentation)
 
 
 class TestNoiseFiltering:
@@ -720,7 +724,7 @@ class TestConvexFill:
         assert np.all(res == arr)
 
     def test__convex_fill(self):
-        arr = np.zeros((20, 20), dtype=np.bool)
+        arr = np.zeros((20, 20), dtype=bool)
         assert _convex_fill(arr) is None
 
 
